package com.xc.config;

import com.alibaba.druid.pool.DruidDataSource;
import com.jfinal.plugin.activerecord.ActiveRecordPlugin;
import com.xc.utils.PropertiesUtil2;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.jdbc.datasource.TransactionAwareDataSourceProxy;

import javax.annotation.PreDestroy;
import java.sql.SQLException;

/**
 * Created on 2020/12/04.
 *
 * @author qlm
 */
@Configuration
public class JfinalActiveRecordConfig implements ApplicationRunner {
    /**
     * 1数据源名称
     */
    private static final String main_DATA_SOURCE_CONFIG = "main";

    /**
     * 2数据源名称
     */
    private static final String data_DATA_SOURCE_CONFIG = "data";


    @Bean(name = "mainDataSource")
    public DruidDataSource mainDataSource()throws SQLException {

        DruidDataSource dataSource = new DruidDataSource();
        dataSource.setUrl(PropertiesUtil2.getProperty("db.url"));
        dataSource.setUsername(PropertiesUtil2.getProperty("db.urlusername"));
        dataSource.setPassword(PropertiesUtil2.getProperty("db.urlpassword"));
        return dataSource;
    }

    @Bean(name = "dataDataSource")
    public DruidDataSource dataDataSource()throws SQLException  {
        DruidDataSource dataSource = new DruidDataSource();
        dataSource.setUrl(PropertiesUtil2.getProperty("db2.url"));
        dataSource.setUsername(PropertiesUtil2.getProperty("db2.username"));
        dataSource.setPassword(PropertiesUtil2.getProperty("db2.password"));

        return dataSource;
    }



    /**
     * 主数据源  main
     * @return
     */
    @Bean
    public ActiveRecordPlugin initmainActiveRecord()  throws SQLException{
        ActiveRecordPlugin arp = new ActiveRecordPlugin(main_DATA_SOURCE_CONFIG, mainTransactionAwareDataSourceProxy());
        arp.start();
        return arp;
    }

    /**
     * 业务数据源  data
     * @return
     */
    @Bean
    public ActiveRecordPlugin initdataActiveRecord()throws SQLException  {
        ActiveRecordPlugin arp = new ActiveRecordPlugin(data_DATA_SOURCE_CONFIG, dataTransactionAwareDataSourceProxy());
        arp.start();
        return arp;
    }




    /**
     * 设置数据源代理
     */
    @Bean
    public TransactionAwareDataSourceProxy mainTransactionAwareDataSourceProxy() throws SQLException{
        TransactionAwareDataSourceProxy transactionAwareDataSourceProxy = new TransactionAwareDataSourceProxy();
        transactionAwareDataSourceProxy.setTargetDataSource(mainDataSource());
        return transactionAwareDataSourceProxy;
    }

    @Bean
    public TransactionAwareDataSourceProxy dataTransactionAwareDataSourceProxy() throws SQLException {
        TransactionAwareDataSourceProxy transactionAwareDataSourceProxy = new TransactionAwareDataSourceProxy();
        transactionAwareDataSourceProxy.setTargetDataSource(dataDataSource());
        return transactionAwareDataSourceProxy;
    }

    /**
     * 设置事务管理
     */
    @Bean(name="mainDataSourceTransactionManager")
    public DataSourceTransactionManager mainDataSourceTransactionManager()  throws SQLException{
        DataSourceTransactionManager dataSourceTransactionManager = new DataSourceTransactionManager();
        dataSourceTransactionManager.setDataSource(mainTransactionAwareDataSourceProxy());
        return dataSourceTransactionManager;
    }

    @Bean(name="dataDataSourceTransactionManager")
    public DataSourceTransactionManager dataDataSourceTransactionManager()  throws SQLException{
        DataSourceTransactionManager dataSourceTransactionManager = new DataSourceTransactionManager();
        dataSourceTransactionManager.setDataSource(dataTransactionAwareDataSourceProxy());
        return dataSourceTransactionManager;
    }




    @Override
    public void run(ApplicationArguments args) throws Exception {
        System.out.println("项目启动啦");
//        String wssurl1 = "ws://127.0.0.1:8080/websocket/zxdata987654+";
//        WsClient myClient = new WsClient(new URI(wssurl1));
//        myClient.connect();
//        // 判断是否连接成功，未成功后面发送消息时会报错
//        while (!myClient.getReadyState().equals(WebSocket.READYSTATE.OPEN)) {
//            System.out.println("连接中···请稍后");
//            Thread.sleep(1000);
//        }

    }

    @PreDestroy
    public void destory() throws Exception {


    }
}
