package com.xc.controller;

import com.xc.common.ServerResponse;
import com.xc.service.IStockService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import javax.servlet.http.HttpServletResponse;

@Controller
@RequestMapping({"/api/stock/"})
public class StockApiController {
    private static final Logger log = LoggerFactory.getLogger(StockApiController.class);

    @Autowired
    IStockService iStockService;

    //获取货币列表
    @RequestMapping({"getStockInfoList.do"})
    @ResponseBody
    public ServerResponse getStockInfoList() {
        return ServerResponse.createBySuccess(iStockService.getStockInfoList());
    }

    //查询 货币货币、大盘货币信息
    @RequestMapping({"getMarket.do"})
    @ResponseBody
    public ServerResponse getMarket() {
        return this.iStockService.getMarket();
    }

    //查询官网PC端交易 所有货币信息及指定货币信息
    @RequestMapping({"getStock.do"})
    @ResponseBody
    public ServerResponse getStock(@RequestParam(value = "num", defaultValue = "0") int num,
                                   @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                   @RequestParam(value = "pageSize", defaultValue = "10") int pageSize,
                                   @RequestParam(value = "stockPlate", required = false) String stockPlate,
                                   @RequestParam(value = "stockType", required = false) String stockType,
                                   @RequestParam(value = "keyWords", required = false) String keyWords) {
        return this.iStockService.getStock(num,pageNum, pageSize, keyWords, stockPlate, stockType);
    }

    //通过货币代码查询货币信息
    @RequestMapping({"getSingleStock.do"})
    @ResponseBody
    public ServerResponse getSingleStock(@RequestParam("id") Integer id) {
        return this.iStockService.getSingleStock(id);
    }

    @RequestMapping({"getMinK.do"})
    @ResponseBody
    public ServerResponse getMinK(@RequestParam("code") String code, @RequestParam("time") Integer time, @RequestParam("ma") Integer ma, @RequestParam("size") Integer size) {
        return this.iStockService.getMinK(code, time, ma, size);
    }

    //查询货币历史数据数据
    @RequestMapping({"getMinK_Echarts.do"})
    @ResponseBody
    public ServerResponse getMinK_Echarts(@RequestParam("code") String code, @RequestParam("time") Integer time, @RequestParam("ma") Integer ma, @RequestParam("size") Integer size) {
        return this.iStockService.getMinK_Echarts(code, time, ma, size);
    }

    //通过货币代码查询货币信息
    @RequestMapping({"getSingleStockConfig.do"})
    @ResponseBody
    public ServerResponse getSingleStockConfig(@RequestParam("id") Integer id) {
        return this.iStockService.getSingleStockConfig(id);
    }
}
