package com.xc.controller.index;


import com.alibaba.fastjson.JSONArray;
import com.xc.common.ServerResponse;
import com.xc.pojo.HistoryIndex;
import com.xc.pojo.Quote;
import com.xc.service.IndexApiService;
import com.xc.vo.index.*;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.List;

/**
 * 货币
 */
@Controller
@RequestMapping({"/api/index/"})
public class IndexApiController {

    private static final Logger log = LoggerFactory.getLogger(IndexApiController.class);

    @Autowired
    private IndexApiService indexApiService;

    // 获取Datafeed 配置数据
    @RequestMapping({"getConfig.do"})
    @ResponseBody
    public Object getConfig() {
        IndexConfigVo indexConfigVo = indexApiService.getConfig();
        return indexConfigVo;
    }


    // 获取商品集合信息
    @RequestMapping({"getSymbolInfo.do"})
    @ResponseBody
    public Object getSymbolInfo() {
        SymbolInfoVo symbolInfoVo = indexApiService.getSymbolInfo();
        return symbolInfoVo;
    }

    // 获取商品解析
    @RequestMapping({"getSymbols.do"})
    @ResponseBody
    public Object getSymbols(@RequestParam String symbol) {
        SymbolVo symbolVo = indexApiService.getSymbols(symbol);
        return symbolVo;
    }

    @RequestMapping(value = {"getHistory.do"}, method = RequestMethod.GET)
    @ResponseBody
    public Object getHistory(@RequestParam String symbol,
                             @RequestParam String resolution,
                             @RequestParam Long from,
                             @RequestParam Long to) {
        HistoryIndex historyIndex = indexApiService.getHistoryNew(symbol.toLowerCase(), resolution, from, to);
        return historyIndex;
    }

    @RequestMapping(value = {"getHistoryNew.do"}, method = RequestMethod.GET)
    @ResponseBody
    public Object getHistoryNew(@RequestParam String symbol,
                                @RequestParam String resolution,
                                @RequestParam Long from,
                                @RequestParam Long to) {
        HistoryIndex historyIndex = indexApiService.getHistoryNew(symbol, resolution, from, to);
        return ServerResponse.createBySuccess(historyIndex);
    }

    //获取服务器时间
    @RequestMapping({"getTime.do"})
    @ResponseBody
    public Object getTime() {
        return System.currentTimeMillis() / 1000;
    }


//    @RequestMapping({"getSearch.do"})
//    @ResponseBody
//    public Object getSearch(@RequestParam String query,
//                            @RequestParam String exchange,
//                            @RequestParam String type,
//                            @RequestParam Long limit) {
//
//        List<SymbolVo> symbolVos = new ArrayList<>();
//        SymbolVo symbolVo = new SymbolVo();
//        symbolVo.setSymbol(IndexEnum.getDbNameByCode(query));
//        symbolVo.setFull_name(String.format("NYSE:%s", IndexEnum.getDbNameByCode(query)));
//        symbolVo.setExchange("NYSE");
//        symbolVo.setTicker("");
//        symbolVos.add(symbolVo);
//        return symbolVos;
//    }


    @RequestMapping({"getMarks.do"})
    @ResponseBody
    public Object getMarks(@RequestParam String symbol,
                           @RequestParam Long from,
                           @RequestParam Long to,
                           @RequestParam String resolution) {
        MarksVO marksVO = new MarksVO();
        marksVO.setId(new Integer[]{0, 1, 2, 3, 4, 5});
        marksVO.setColor(new String[]{"red", "blue", "green", "red", "blue", "green"});
        marksVO.setLabel(new String[]{"A", "B", "CORE", "D", "EURO", "F"});
        marksVO.setLabelFontColor(new String[]{"white", "white", "red", "#FFFFFF", "white", "#000"});
        marksVO.setMinSize(new String[]{""});
        marksVO.setText(new String[]{});
        marksVO.setTime(new Integer[]{});
        return marksVO;
    }


    @RequestMapping({"getTimescaleMarks.do"})
    @ResponseBody
    public Object getTimescaleMarks(@RequestParam String symbol,
                                    @RequestParam Long from,
                                    @RequestParam Long to,
                                    @RequestParam String resolution) {
        List<TimescaleMarks> list = new ArrayList<>();
        TimescaleMarks timescaleMarks = new TimescaleMarks();
        timescaleMarks.setColor("red");
        timescaleMarks.setId("tsm1");
        timescaleMarks.setLabel("A");
        timescaleMarks.setTime(System.currentTimeMillis() / 1000);
        timescaleMarks.setTooltip("");
        list.add(timescaleMarks);
        return list;
    }

    //获取报价
    @RequestMapping({"getQuotes.do"})
    @ResponseBody
    public Object getQuotes(@RequestParam String symbols) {
        List<Quote> quotes = indexApiService.getQuotes(symbols);

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("s", "ok");
        jsonObject.put("n", "NYSE:AA");
        jsonObject.put("v", quotes.get(0));

        JSONObject jsonObject2 = new JSONObject();
        jsonObject2.put("s", "ok");
        jsonObject2.put("n", "NYSE:AA");
        jsonObject2.put("v", quotes.get(0));


        JSONArray jsonArray = new JSONArray();
        jsonArray.add(jsonObject);
        jsonArray.add(jsonObject2);


        JSONObject jsonObject1 = new JSONObject();
        jsonObject1.put("s", "ok");
        jsonObject1.put("d", jsonArray);
        return ServerResponse.createBySuccess(jsonObject1);
    }
}
