package com.xc.controller.news;

import com.github.pagehelper.PageInfo;
import com.xc.common.ServerResponse;
import com.xc.pojo.News;
import com.xc.service.NewsService;
import com.xc.utils.thread.ThreadUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

@Slf4j
@RestController
@RequestMapping({"/admin/news/"})
public class AdminNewsApiController {
    @Autowired
    NewsService newsService;

    @RequestMapping("getApiNews.do")
    public ServerResponse getApiNews(Integer num, Integer start,Integer max){
        ThreadUtil.executeCallback(obj -> {
            for (int i = start; i < max; i++) {
                newsService.getApiNews(num,i);
            }
        },0,5000);
        return ServerResponse.createBySuccess();
    }

    /**
     * 新闻列表
     *
     * @param news
     * @param pageNum  当前第几页
     * @param pageSize 每页个数
     * @return
     */
    @RequestMapping(value = "list.do", method = {RequestMethod.GET, RequestMethod.POST})
    public ServerResponse<PageInfo> list(News news, @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum, @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        PageInfo pageInfo = newsService.listByNews(news, pageNum, pageSize);
        return ServerResponse.createBySuccess(pageInfo);
    }

    @RequestMapping(value = "add.do")
    public ServerResponse add(News news,Long publishTimeLong) {
        if (publishTimeLong!=null){
            Date date = new Date(publishTimeLong);
            news.setPublishTime(date);
        }
        Integer updateCount = newsService.insert(news);
        if (updateCount > 0) {
            return ServerResponse.createBySuccessMsg("新增成功");
        }
        return ServerResponse.createByErrorMsg("新增失败");
    }

    @RequestMapping(value = "update.do")
    public ServerResponse update(News news,Long publishTimeLong) {
        if (publishTimeLong!=null){
            Date date = new Date(publishTimeLong);
            news.setPublishTime(date);
        }
        Integer updateCount = newsService.updateByPrimaryKeySelective(news);
        if (updateCount > 0) {
            return ServerResponse.createBySuccessMsg("修改成功");
        }
        return ServerResponse.createByErrorMsg("修改失败");
    }

    @RequestMapping(value = "delete.do")
    public ServerResponse delete(Integer id) {
        Integer updateCount = newsService.deleteByPrimaryKey(id);
        if (updateCount > 0) {
            return ServerResponse.createBySuccessMsg("删除成功");
        }
        return ServerResponse.createByErrorMsg("删除失败");
    }

    @RequestMapping({"select.do"})
    public ServerResponse select(Integer id) {
        News updateCount = newsService.selectByPrimaryKey(id);
        return ServerResponse.createBySuccess(updateCount);
    }


}
