package com.xc.controller.ueditor;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.xc.utils.PropertiesUtil;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.util.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;

@RestController
@RequestMapping("/ueditor")
public class UeditorConfigController {

    @Value("classpath:data/ueditorConfig.json")
    private Resource resource;

    private static String imageUrlPrefix = PropertiesUtil.getProperty("ueditor.config.imageUrlPrefix");

    //@Value("${web.file.path}")
//    private String filePath;
    /**
     * 请求根地址
     * uploadimage：//执行上传图片或截图的action名称
     * uploadscrawl：//执行上传涂鸦的action名称
     * uploadvideo：//执行上传视频的action名称
     * uploadfile：//controller里,执行上传视频的action名称
     * catchimage：//执行抓取远程图片的action名称
     * listimage：//执行列出图片的action名称
     * listfile：//执行列出文件的action名称
     * ,produces="text/plain;charset=utf-8"
     *
     * @param request
     * @param response
     * @param action
     * @return
     */
    @RequestMapping("/config.do")
    public Object getConfig(HttpServletRequest request,HttpServletResponse response, HttpSession session,
                            String action) {
        JSONObject result = new JSONObject();
        response.setCharacterEncoding("utf-8");
        response.setHeader("Access-Control-Allow-Origin", "*");

        //  , @RequestParam(value = "upfile", required = false) MultipartFile upfile
        MultipartFile upfile= null;

        switch (action) {
            case "config":
                result = getJsonObjFromResource();
                result.put("imageUrlPrefix",imageUrlPrefix);
                break;
                //执行上传图片或截图的action名称
            case "uploadimage":
                //执行上传涂鸦的action名称
            case "uploadscrawl":
                try {
                    boolean isMultipart = ServletFileUpload.isMultipartContent(request);
                    //判断是否为文件上传
                    if (isMultipart){
                        MultipartHttpServletRequest multipartRequest = WebUtils.getNativeRequest(request, MultipartHttpServletRequest.class);
                        upfile = multipartRequest.getFile("upfile");
                        if (null != upfile){
                            result = this.ueditorUpload(upfile, session);
                        }else {
                            result.put("state","文件上传错误");
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
                break;
            default:
                result.put("state","参数错误");
        }
        return result;
    }

    /**
     * @param @param  file
     * @param @param  response
     * @param @param  request
     * @param @throws Exception
     * @return void
     * @Title: uploadUEditorImage
     * @Description: 自定义编辑器上传路径
     * @date createTime：2018年4月9日下午1:52:19
     */
    @RequestMapping("/ueditorUpload.do")
    public JSONObject ueditorUpload(@RequestParam(value = "upfile", required = false) MultipartFile file, HttpSession session) throws Exception {
        JSONObject json = new JSONObject();
        try {
            //旧文件名
            String fileName = file.getOriginalFilename();
            //新文件名
            String name = System.currentTimeMillis() + "" + (int) (Math.random() * 100);
            //创建以日期为名字的文件夹
            String dateFolder = "/" + new SimpleDateFormat("yyyyMMdd").format(new Date()) + "/";
            String filePathString = session.getServletContext().getRealPath("/static/upImages" + dateFolder);
            File filePath = this.savaImg(file, filePathString, name);
            json.put("state", "SUCCESS");
            json.put("title", fileName);
            // 图片访问 相对路径
            json.put("url", session.getServletContext().getContextPath() + "/static/upImages" + dateFolder + filePath.getName());
            json.put("original", fileName);
        } catch (Exception e) {
            json.put("state", "ERROR"+e);
        }
        return json;
    }

    public File savaImg(MultipartFile file, String filePath, String name) {
        File fileUrl = null;
        if (!file.isEmpty()) {
            String originalFilename = file.getOriginalFilename();
            String nextfix = originalFilename.split("\\.")[originalFilename.split("\\.").length - 1];
            if ("jpg".equals(nextfix) || "png".equals(nextfix) || "gif".equals(nextfix) || "jpeg".equals(nextfix)) {
                String newFileName = name + "." + nextfix;
                File testFilePath = new File(filePath);//用来判断文件夹是否存在
                File imgFilePath = new File(filePath + "/" + newFileName);//生成文件文件上传路径
                try {
                    if (!testFilePath.exists()) {
                        testFilePath.mkdirs();
                    }
                    file.transferTo(imgFilePath);
                    fileUrl = imgFilePath;
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return fileUrl;
    }

    public  JSONObject getJsonObjFromResource() {
        JSONObject json = null;
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(resource.getInputStream()));
            StringBuffer message = new StringBuffer();
            String line = null;
            while ((line = br.readLine()) != null) {
                message.append(line);
            }
            String defaultString = message.toString();
            json = JSON.parseObject(defaultString);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return json;
    }


}
