package com.xc.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.xc.dao.IndexApiMapper;
import com.xc.dao2.KLinkStockMapper;
import com.xc.pojo.HistoryIndex;
import com.xc.pojo.Quote;
import com.xc.vo2.HistoryVo;
import com.xc.pub.redismessage.util.RedisUtil;
import com.xc.service.IndexApiService;
import com.xc.utils.enumUtil.ResolutionEnum;
import com.xc.vo.index.IndexConfigVo;
import com.xc.vo.index.SymbolInfoVo;
import com.xc.vo.index.SymbolVo;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Service("IndexApiService")
public class IndexApiServiceImpl implements IndexApiService {

    private static final Logger log = LoggerFactory.getLogger(IndexApiServiceImpl.class);

    @Autowired
    private IndexApiMapper indexApiMapper;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private KLinkStockMapper kLinkStockMapper;

    @Override
    public IndexConfigVo getConfig() {
        IndexConfigVo indexConfigVo = new IndexConfigVo();
        indexConfigVo.setSupports_search(true);

        indexConfigVo.setSupported_resolutions(ResolutionEnum.getNameList());
        indexConfigVo.setSupports_marks(false);
        indexConfigVo.setSupports_time(false);
        indexConfigVo.setSupports_timescale_marks(false);
        List<IndexConfigVo.Exchange> exchanges = new ArrayList<>();
        IndexConfigVo.Exchange exchange = new IndexConfigVo.Exchange();
        exchange.setDesc("NYSE");
        exchange.setName("NYSE");
        exchange.setValue("NYSE");
        exchanges.add(exchange);
        indexConfigVo.setExchanges(exchanges);

        List<IndexConfigVo.SymbolsType> symbolsTypes = new ArrayList<>();
        IndexConfigVo.SymbolsType symbolsType = new IndexConfigVo.SymbolsType();
        symbolsType.setName("Index");
        symbolsType.setValue("index");
        symbolsTypes.add(symbolsType);
        indexConfigVo.setSymbols_types(symbolsTypes);
        return indexConfigVo;
    }

    @Override
    public SymbolInfoVo getSymbolInfo() {
        SymbolInfoVo symbolInfoVo = new SymbolInfoVo();
        symbolInfoVo.setExchange_listed("NYSE");
        symbolInfoVo.setExchange_traded("NYSE");
        symbolInfoVo.setMinmov(1);
        symbolInfoVo.setMinmov2(0);
        symbolInfoVo.setHas_intraday(true);
        symbolInfoVo.setIntraday_multipliers(Arrays.asList("1", "5", "15", "30", "60", "240", "1D", "1W", "1M", "6M", "12M"));
        symbolInfoVo.setTimezone("Asia/Shanghai");
        symbolInfoVo.setSession_regular("0900-1600");
        return symbolInfoVo;
    }

    @Override
    public SymbolVo getSymbols(String symbol) {
        SymbolVo symbolVo = new SymbolVo();
        symbolVo.setSymbol(symbol);
        symbolVo.setDescription(symbol);
        symbolVo.setExchange_listed("NYSE");
        symbolVo.setExchange_traded("NYSE");
        symbolVo.setMinmov(1);
        symbolVo.setMinmov2(0);
        symbolVo.setPricescale(100);
        symbolVo.setPointvalue(1);
        symbolVo.setHas_intraday(true);
        symbolVo.setIntraday_multipliers(Arrays.asList("1", "5", "15", "30", "60"));
        symbolVo.setHas_daily(true);
        symbolVo.setHas_weekly_and_monthly(true);
        symbolVo.setType(symbol);
        symbolVo.setTicker(symbol);
        symbolVo.setTimezone("Asia/Shanghai");
        symbolVo.setSession_regular("0900-1600");
        symbolVo.setSupported_resolutions(ResolutionEnum.getNameList());
        symbolVo.setHas_no_volume(true);
        return symbolVo;
    }

//    @Override
//    public RealTimeIndex getRealTimeData(String indexCode, Date time) {
//        String dbTableName = IndexDbCreatedUtil.createdDbTableName(indexCode, time);
//        return indexApiMapper.getRealTimeData(dbTableName);
//    }
//
//    @Override
//    public List<RealTimeIndex> getHistData(String indexCode, Date time) {
//        String dbTableName = IndexDbCreatedUtil.createdHistDbTableName(indexCode);
//        return indexApiMapper.getHistData(dbTableName, time);
//    }

    @Override
    public List<Quote> getQuotes(String symbols) {
        List<Quote> list = new ArrayList<>();

        Quote quote = new Quote();
        quote.setCh("+0.16");
        quote.setChp(0.98);
        quote.setShortName("NYSE:AA");
        quote.setDescription("Alcoa Inc. Common");
        quote.setLp(16.57);
        //模拟数据
        quote.setVolume(4029041);

        list.add(quote);
        return list;
    }

    private List<HistoryVo> remove(List<HistoryVo> historyVoList) {
        Set<HistoryVo> set = new TreeSet<>(new Comparator<HistoryVo>() {
            @Override
            public int compare(HistoryVo r1, HistoryVo r2) {
                //字符串,则按照Asicc码升序排列
                return r1.getId().compareTo(r2.getId());
            }
        });
        set.addAll(historyVoList);
        return new ArrayList<>(set);
    }

    private int getMinute(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.get(Calendar.MINUTE);
    }

    private Long getDate(Long time, Integer type) {
        try {
            // 将秒转成毫秒
            Date date = new Date(time);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            // 时间先取整
            if(1 == type) {
                sdf = new SimpleDateFormat("yyyyMMddHHmm");
            }else {
                sdf = new SimpleDateFormat("yyyyMMdd");
            }
            String dateStr = sdf.format(date);
            date = sdf.parse(dateStr);
            return date.getTime() / 1000; // 将毫秒数转换成秒
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return time / 1000;
    }

    private void getDate(String symbol, String resolution, Long from, Long to, List<HistoryVo> historyVoList, List<HistoryVo> historyVoDays){
        if(isIntraday(resolution)){
            int minuteTo = getMinute(new Date(to * 1000));
            int resolutionInt = Integer.valueOf(resolution);
            if(0 != minuteTo%resolutionInt){
                to = getDate(to * 1000, 1) + ((resolutionInt - minuteTo%resolutionInt) * 60);
            }
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
        Integer fromX = Integer.valueOf(sdf.format(from));
        Integer newX = Integer.valueOf(sdf.format(new Date()));
        String dbTableName;
        String redisKey = String.format("data_division_%s_%s", symbol, resolution);
//        String redisData = redisUtil.getItem(redisKey);
        String redisData = null;
        if (StringUtils.isNotEmpty(redisData) && !("[]".equals(redisData))) {
            // 反序列化
            List<HistoryVo> historyVoRedis = JSONArray.parseArray(redisData, HistoryVo.class);
            if (!CollectionUtils.isEmpty(historyVoRedis)) {
                // redis里面肯定没有最新的数据 所以还需要去数据库取
//                Collections.sort(historyVoRedis); // 按时间倒序排序
                // 取第一条 判断时间是否在请求from的时间之前 如果在时间之后表示redis中的数据不覆盖实际需要的数据
                Long startTime = historyVoRedis.get(0).getTime();
                Long entTime = historyVoRedis.get(historyVoRedis.size() - 1).getTime();
                if(from > startTime && to > entTime){
                    // 表示只包含开始的 没有包含结束的 所以要去取后面的数据
                    Long realTime = historyVoRedis.get(historyVoRedis.size() - 1).getTime();
                    if(isIntraday(resolution)) {
                        dbTableName = String.format("data_rt_%s", symbol);
                        historyVoRedis.addAll(kLinkStockMapper.getDataByRt(dbTableName, realTime, to, resolution));
                    }else{
                        dbTableName = String.format("data_hist_%s", symbol);
                        historyVoRedis.addAll(kLinkStockMapper.getDataByHist(dbTableName, realTime, to, resolution));
                    }
                } else if(from < startTime && to > entTime){
                    // 全部不包含
                    if (fromX.equals(newX)) {
                        // 表示在同一年
                        if (fromX >= 2020) {
                            if(isIntraday(resolution)) {
                                dbTableName = String.format("data_rt_%s", symbol);
                                historyVoRedis.addAll(kLinkStockMapper.getDataByRt(dbTableName, from, to, resolution));
                            }else{
                                dbTableName = String.format("data_hist_%s", symbol);
                                historyVoRedis.addAll(kLinkStockMapper.getDataByHist(dbTableName, from, to, resolution));
                            }
                        }
                    } else {
                        // 将跨度内的都取出来
                        if(isIntraday(resolution)) {
                            for (int i = fromX; i <= newX; i++) {
                                if (i >= 2020) {
                                    dbTableName = String.format("data_rt_%s",symbol);
                                    historyVoRedis.addAll(kLinkStockMapper.getDataByRt(dbTableName, from, to, resolution));
                                }
                            }
                        }else{
                            dbTableName = String.format("data_hist_%s", symbol);
                            historyVoRedis.addAll(kLinkStockMapper.getDataByHist(dbTableName, from, to, resolution));
                        }
                    }
                }
                // 去掉重复数据
                historyVoRedis = remove(historyVoRedis);
            }
            // 去除需要的数据
            for (HistoryVo historyVo : historyVoRedis) {
                if (historyVo.getTime() >= from && historyVo.getTime() <= to) {
                    historyVoList.add(historyVo);
                }
            }
//            Collections.sort(historyVoList); // 按时间倒序排序
//            redisUtil.deleteItem(redisKey);
            redisUtil.saveItem(redisKey, JSON.toJSONString(historyVoRedis), 5400, TimeUnit.SECONDS);
        }
        else{
            // redis中没有一条数据
            if (isIntraday(resolution)) {
                if (fromX.equals(newX)) {
                    // 表示在同一年
                    if (fromX >= 2020) {
                        dbTableName = String.format("data_rt_%s", symbol);
                        historyVoList.addAll(kLinkStockMapper.getDataByRt(dbTableName, from, to, resolution));
                    }
                } else {
                    // 将跨度内的都取出来
                    for (int i = fromX; i <= newX; i++) {
                        if (i >= 2020) {
                            dbTableName = String.format("data_rt_%s", symbol);
                            historyVoList.addAll(kLinkStockMapper.getDataByRt(dbTableName, from, to, resolution));
                        }
                    }
                }
            } else {
                dbTableName = String.format("data_hist_%s", symbol);
                historyVoList.addAll(kLinkStockMapper.getDataByHist(dbTableName, from, to, resolution));
            }
        }
        historyVoList = remove(historyVoList);
//        Collections.sort(historyVoList); // 按时间倒序排序
//        redisUtil.deleteItem(redisKey);
        redisUtil.saveItem(redisKey, JSON.toJSONString(historyVoList), 5400, TimeUnit.SECONDS);
        // 日表中取最后一天的数据 最后一天的数据历史表中要到第二天才有 所以取实时表的数据
        if(!CollectionUtils.isEmpty(historyVoList) || getDate(new Date().getTime() / 1000, 0, resolution).equals(getDate(new Date().getTime() / 1000, 0, resolution))) {
            if (resolution.equals("1D")) {
                if (newX >= 2020) {
                    dbTableName = String.format("data_rt_%s", symbol);
                    String time = getDate(new Date().getTime() / 1000, 0, resolution);
                    historyVoDays.addAll(kLinkStockMapper.getHistoryNew(dbTableName, Long.valueOf(time), Long.valueOf(getDate(to, 1, resolution))));
                }
            }
        }
    }

    @Override
    public HistoryIndex getHistoryNew(String symbol, String resolution, Long from, Long to) {
        if(from > to){
            to = new Date().getTime() / 1000;
        }
        List<HistoryVo> historyVoList = new ArrayList<>();
        List<HistoryVo> historyVoDays = new ArrayList<>();
        getDate(symbol, resolution, from, to, historyVoList, historyVoDays);
//        Collections.sort(historyVoList); // 按时间倒序排序
        HistoryIndex historyIndex = new HistoryIndex();
        if (CollectionUtils.isEmpty(historyVoList) && CollectionUtils.isEmpty(historyVoDays)) {
            historyIndex.setS("no_data");
            // A50的数据起始是2016年 其他的都是1990年
            Calendar cal = Calendar.getInstance();
            cal.setTime(new Date(from * 1000));
            int year = cal.get(Calendar.YEAR);
            if (!isIntraday(resolution)&& 1990 < year) {
                historyIndex.setNextTime(System.currentTimeMillis() / 1000);
            } else if (isIntraday(resolution)) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                Date date = null;
                try {
                    date = sdf.parse("2020-09-01");
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                if (date.before(new Date(from * 1000))) {
                    historyIndex.setNextTime(System.currentTimeMillis() / 1000);
                }
            }
            return historyIndex;
        }
        historyIndex.setS("ok");
        // 数据拼接
        Double[][] arr = new Double[8][historyVoList.size() + 1];
        int i = 0;
        if(!CollectionUtils.isEmpty(historyVoList)) {
            for (HistoryVo historyVo : historyVoList) {
                // 过滤垃圾数据
                if (null != historyVo.getRealTime() && null != historyVo.getClose()
                        && null != historyVo.getOpen() && null != historyVo.getHigh() && null != historyVo.getLow()
                        && null != historyVo.getVolume()) {
                    arr[0][i] = Double.valueOf(historyVo.getTime());
                    arr[1][i] = historyVo.getClose();
                    arr[2][i] = historyVo.getOpen();
                    arr[3][i] = historyVo.getHigh();
                    arr[4][i] = historyVo.getLow();
                    arr[5][i] = historyVo.getVolume();
                    arr[6][i] = Double.valueOf(historyVo.getId());
                    arr[7][i] = Double.valueOf(historyVo.getRealTime());
                    i++;
                }
            }
        }
        if(!CollectionUtils.isEmpty(historyVoDays)) {
            HistoryVo temp = historyVoDays.get(0);
            temp.setOpen(temp.getClose());
            temp.setLow(temp.getClose());
            temp.setHigh(temp.getClose());
            temp.setVolume(new Double(0));
            for (HistoryVo historyVo : historyVoDays) {
                // 过滤垃圾数据
                if (null != historyVo.getRealTime() && null != historyVo.getClose()
                        && null != historyVo.getOpen() && null != historyVo.getHigh() && null != historyVo.getLow()
                        && null != historyVo.getVolume()) {
                    // 找到最高价
                    if (temp.getHigh() < historyVo.getClose()) {
                        temp.setHigh(historyVo.getClose());
                    }
                    // 找到最低价格
                    if (temp.getLow() > historyVo.getClose()) {
                        temp.setLow(historyVo.getClose());
                    }
                    temp.setRealTime(historyVo.getRealTime());
                    temp.setClose(historyVo.getClose());
                    temp.setVolume(temp.getVolume() + historyVo.getVolume());
                }
            }
            String time = getDate(temp.getTime(), 0, resolution);
            arr[0][i] = Double.valueOf(temp.getTime());
            arr[1][i] = temp.getClose();
            arr[2][i] = temp.getOpen();
            arr[3][i] = temp.getHigh();
            arr[4][i] = temp.getLow();
            arr[5][i] = temp.getVolume();
            arr[6][i] = Double.valueOf(temp.getId());
            arr[7][i] = Double.valueOf(time);
            i++;
        }
        List<Double> t = Arrays.asList(arr[0]);
        List t1 = new ArrayList(t);
        t1.removeAll(Collections.singleton(null));
        historyIndex.setT(t1);

        List<Double> c = Arrays.asList(arr[1]);
        List c1 = new ArrayList(c);
        c1.removeAll(Collections.singleton(null));
        historyIndex.setC(c1);

        List<Double> o = Arrays.asList(arr[2]);
        List o1 = new ArrayList(o);
        o1.removeAll(Collections.singleton(null));
        historyIndex.setO(o1);

        List<Double> h = Arrays.asList(arr[3]);
        List h1 = new ArrayList(h);
        h1.removeAll(Collections.singleton(null));
        historyIndex.setH(h1);

        List<Double> l = Arrays.asList(arr[4]);
        List l1 = new ArrayList(l);
        l1.removeAll(Collections.singleton(null));
        historyIndex.setL(l1);

        List<Double> v = Arrays.asList(arr[5]);
        List v1 = new ArrayList(v);
        v1.removeAll(Collections.singleton(null));
        historyIndex.setV(v1);

        List<Double> id = Arrays.asList(arr[6]);
        List id1 = new ArrayList(id);
        id1.removeAll(Collections.singleton(null));
        historyIndex.setId(id1);

        List<Double> time = Arrays.asList(arr[7]);
        List time1 = new ArrayList(time);
        time1.removeAll(Collections.singleton(null));
        historyIndex.setTime(time1);
        return historyIndex;
    }

    private static boolean isIntraday(String resolution) {
        if (resolution.equals("1") || resolution.equals("5") || resolution.equals("15") || resolution.equals("30")
                || resolution.equals("60")) {
            return true;
        }
        return false;
    }

    private static Date getDate(Date date, long day, String resolution) {
        try {
            // 时间先取整
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            if (isIntraday(resolution)) {
                sdf = new SimpleDateFormat("yyyyMMddHHmm");
            } else if (resolution.equals("1D") || resolution.equals("1W")) {
                sdf = new SimpleDateFormat("yyyyMMdd");
            } else if (resolution.equals("1M")) {
                sdf = new SimpleDateFormat("yyyyMM");
            } else if (resolution.equals("1Y") || resolution.equals("M")) {
                sdf = new SimpleDateFormat("yyyy");
            }
            String dateStr = sdf.format(date);
            date = sdf.parse(dateStr);
            long time = date.getTime(); // 得到指定日期的毫秒数
            day = day * 24 * 60 * 60 * 1000; // 要加上的天数转换成毫秒数
            time += day; // 相加得到新的毫秒数
            return new Date(time); // 将毫秒数转换成日期
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }

    private static String getDate(Long timeStr, long day, String resolution) {
        try {
            // 将秒转成毫秒
            timeStr = timeStr * 1000;
            Date date = new Date(Long.valueOf(timeStr));
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            // 时间先取整
            if (isIntraday(resolution)) {
                sdf = new SimpleDateFormat("yyyyMMddHHmm");
            } else if (resolution.equals("1D") || resolution.equals("1W")) {
                sdf = new SimpleDateFormat("yyyyMMdd");
            } else if (resolution.equals("1M")) {
                sdf = new SimpleDateFormat("yyyyMM");
            } else if (resolution.equals("1Y") || resolution.equals("M")) {
                sdf = new SimpleDateFormat("yyyy");
            }
            String dateStr = sdf.format(date);
            date = sdf.parse(dateStr);
            long time = date.getTime(); // 得到指定日期的毫秒数
            day = day * 24 * 60 * 60 * 1000; // 要加上的天数转换成毫秒数
            time += day; // 相加得到新的毫秒数
            return String.valueOf(new Date(time).getTime() / 1000); // 将毫秒数转换成秒
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return timeStr.toString();
    }

    private static Integer getTime(String timeStr, String resolution) {
        timeStr = timeStr + "000";
        Date date = new Date(Long.valueOf(timeStr));
        Integer returnTime = null;
        if (isIntraday(resolution)) {
            if("60".equals(resolution)){
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHH");
                String dateStr = sdf.format(date);
                returnTime = Integer.valueOf(dateStr.substring(dateStr.length() - 2));
            }else {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmm");
                String dateStr = sdf.format(date);
                int m = Integer.valueOf(dateStr.substring(dateStr.length() - 2));
                // 取最后两位
                returnTime = m / Integer.valueOf(resolution);
            }
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            Calendar calendar = Calendar.getInstance();
            try {
                String dateStr = sdf.format(date);
                date = sdf.parse(dateStr);
                calendar.setTime(date);
                if (resolution.equals("1D")) {
                    returnTime = calendar.get(Calendar.DAY_OF_YEAR);// 一年中的第几天
                } else if (resolution.equals("1W")) {
                    returnTime = calendar.get(Calendar.WEEK_OF_YEAR); // 一年中的第几周
                } else if (resolution.equals("1M")) {
                    returnTime = calendar.get(Calendar.MONTH); // 第几个月
                } else if (resolution.equals("1Y") || resolution.equals("M")) {
                    returnTime = calendar.get(Calendar.YEAR); // 年份数值
                }
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }
        return returnTime;
    }

    private static Long getTime(Date date, String resolution) {
        if (resolution.equals("1M")) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMM");
            String dateStr = sdf.format(date);
            return Long.valueOf(dateStr);
        } else if (resolution.equals("1Y") || resolution.equals("M")) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
            String dateStr = sdf.format(date);
            return Long.valueOf(dateStr);
        }
        return null;
    }
}
