package com.xc.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xc.common.ServerResponse;
import com.xc.dao.ProtocolMapper;
import com.xc.dao.SiteAdminMapper;
import com.xc.dao.UserMapper;
import com.xc.pojo.ProtocolInfo;
import com.xc.pojo.ProtocolType;
import com.xc.pojo.SiteAdmin;
import com.xc.service.ProtocolService;
import com.xc.utils.HttpClientRequest;
import com.xc.utils.PropertiesUtil;
import com.xc.utils.enumUtil.ProtocolTypeEnum;
import com.xc.utils.redis.CookieUtils;
import com.xc.utils.redis.JsonUtil;
import com.xc.utils.redis.RedisShardedPoolUtils;
import com.xc.vo.protocol.ProtocolAdminVo;
import com.xc.vo.protocol.ProtocolApiVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service("ProtocolService")
public class ProtocolServiceImpl implements ProtocolService {

    @Autowired
    private ProtocolMapper protocolMapper;

    @Resource
    private SiteAdminMapper siteAdminMapper;
    @Resource
    private UserServiceImpl userService;

    @Override
    public List<ProtocolType> getProtocolType() {
        return ProtocolTypeEnum.getList();
    }

    @Override
    public ProtocolApiVo getProtocolInfoByType(String typeId) {
        ProtocolApiVo protocolApiVo = new ProtocolApiVo();
        ProtocolInfo protocolInfo = protocolMapper.selectProtocolInfoByType(typeId);
        if(null == protocolInfo){
            return null;
        }
        BeanUtils.copyProperties(protocolInfo, protocolApiVo);
        return protocolApiVo;
    }

    @Override
    public PageInfo getProtocolInfoList(String protocolName, String typeId,Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum,pageSize);
        List<ProtocolAdminVo> protocolAdminVoList = new ArrayList<>();
        List<ProtocolInfo> protocolInfoList = protocolMapper.selectProtocolInfo(protocolName, typeId);
        ProtocolAdminVo protocolAdminVo;
        for(ProtocolInfo protocolInfo : protocolInfoList){
            protocolAdminVo = new ProtocolAdminVo();
            BeanUtils.copyProperties(protocolInfo, protocolAdminVo);
            protocolAdminVoList.add(protocolAdminVo);
        }
        PageInfo pageInfo = new PageInfo(protocolAdminVoList);
        return pageInfo;
    }

    @Override
    public ServerResponse addProtocol(ProtocolInfo protocolInfo, HttpServletRequest request) {
        // 登录用户
        String loginToken = CookieUtils.readLoginToken(request, PropertiesUtil.getProperty("admin.cookie.name"));
        String adminJsonStr = RedisShardedPoolUtils.get(loginToken);
        SiteAdmin siteAdmin = (SiteAdmin) JsonUtil.string2Obj(adminJsonStr, SiteAdmin.class);
        StringBuffer stringBuffer=new StringBuffer();
        stringBuffer.append(siteAdmin.getAdminName());
        stringBuffer.append("新增数据");
        Date date=new Date();
        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        stringBuffer.append(f.format(date));
        //String remark = admin + "新增数据" + new Date();
        protocolInfo.setRemark(stringBuffer.toString());
        protocolInfo.setStatus(0);
        int insertcount=protocolMapper.insert(protocolInfo);
        if (insertcount > 0)
            return ServerResponse.createBySuccessMsg("添加成功");

        return ServerResponse.createByErrorMsg("添加失败");
    }

    @Override
    public ServerResponse updateProtocol(ProtocolInfo protocolInfo,HttpServletRequest request) {
        String loginToken = CookieUtils.readLoginToken(request, PropertiesUtil.getProperty("admin.cookie.name"));
        String adminJsonStr = RedisShardedPoolUtils.get(loginToken);
        SiteAdmin siteAdmin = (SiteAdmin) JsonUtil.string2Obj(adminJsonStr, SiteAdmin.class);
        StringBuffer stringBuffer=new StringBuffer();
        stringBuffer.append(siteAdmin.getAdminName());
        stringBuffer.append("修改数据");
        Date date=new Date();
        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        stringBuffer.append(f.format(date));
        //String remark = admin + "新增数据" + new Date();
        protocolInfo.setRemark(stringBuffer.toString());
        protocolInfo.setStatus(1);

        int updatecount=protocolMapper.update(protocolInfo);

        if (updatecount > 0)
            return ServerResponse.createBySuccessMsg("修改成功");
        return ServerResponse.createByErrorMsg("修改失败");
    }

    @Override
    public ProtocolInfo selectByid(Integer param) {
        return protocolMapper.selectByid(param);
    }

    @Override
    public Integer deleteProtocol(Integer id) {
        return protocolMapper.delete(id);
    }
}
