package com.xc.service.impl;


import com.xc.common.ServerResponse;
import com.xc.dao.SiteInfoMapper;
import com.xc.pojo.SiteInfo;
import com.xc.service.ISiteInfoService;
import com.xc.utils.PropertiesUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


@Service("iSiteInfoService")

public class SiteInfoServiceImpl implements ISiteInfoService {
    @Autowired
    SiteInfoMapper siteInfoMapper;


    public ServerResponse<SiteInfo> get() {

        List<SiteInfo> siteInfos = this.siteInfoMapper.findAll();
        if (siteInfos.size() > 0) {
            SiteInfo siteInfo = (SiteInfo) siteInfos.get(0);
            return ServerResponse.createBySuccess(siteInfo);
        }
        return ServerResponse.createByErrorMsg("设置信息不存在");

    }


    public ServerResponse add(SiteInfo siteInfo) {

        List<SiteInfo> siteInfos = this.siteInfoMapper.findAll();

        if (siteInfos.size() > 0) {
            return ServerResponse.createByErrorMsg("不能重复添加");
        }

        if (StringUtils.isBlank(siteInfo.getSiteName()) ||
                StringUtils.isBlank(siteInfo.getSiteLogo()) ||
                StringUtils.isBlank(siteInfo.getSiteLogoSm())) {
            return ServerResponse.createByErrorMsg("名字和logo不能为空");
        }

        int insertCount = this.siteInfoMapper.insert(siteInfo);
        if (insertCount > 0) {
            return ServerResponse.createBySuccessMsg("添加成功");
        }
        return ServerResponse.createByErrorMsg("添加失败");

    }


    public ServerResponse update(SiteInfo siteInfo) {

        if (siteInfo.getId() == null) {
            return ServerResponse.createByErrorMsg("ID不能为空");
        }

        int updateCount = this.siteInfoMapper.updateByPrimaryKeySelective(siteInfo);

        if (updateCount > 0) {
            return ServerResponse.createBySuccessMsg("修改成功");
        }
        return ServerResponse.createByErrorMsg("修改失败");
    }

    public ServerResponse<SiteInfo> getInfo() {
        try {
            List<SiteInfo> siteInfos = this.siteInfoMapper.findAll();
            if (siteInfos.size() > 0) {
                SiteInfo siteInfo = (SiteInfo) siteInfos.get(0);
                String isValidationSms = PropertiesUtil.getProperty("is.validation.sms");
                if (StringUtils.isBlank(isValidationSms) || "true".equals(isValidationSms)) {
                    siteInfo.setValidationSms(true);
                } else {
                    siteInfo.setValidationSms(false);
                }
                return ServerResponse.createBySuccess(siteInfo);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ServerResponse.createByErrorMsg("设置信息info不存在");
    }
}
