package com.xc.service.impl;

import com.xc.dao.StockMapper;
import com.xc.dao.StockMarketsDayMapper;
import com.xc.pojo.Stock;
import com.xc.pojo.StockMarketsDay;
import com.xc.service.DbStockService;
import com.xc.service.IStockMarketsDayService;
import com.xc.service.IStockService;
import com.xc.service.IUserPositionService;
import com.xc.utils.DateTimeUtil;
import com.xc.vo.stock.StockListVO;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("iStockMarketsDayService")
public class StockMarketsDayServiceImpl implements IStockMarketsDayService {

    private static final Logger log = LoggerFactory.getLogger(StockMarketsDayServiceImpl.class);

    @Autowired
    IStockService iStockService;

    @Autowired
    StockMarketsDayMapper stockMarketsDayMapper;

    @Autowired
    private StockMapper stockMapper;

    @Autowired
    private IUserPositionService userPositionService;

    @Autowired
    DbStockService dbStockService;

    public void saveStockMarketDay() {
        log.info("【保存货币日内行情 定时任务】 开始保存 ... ");
        List<Stock> stockList = this.iStockService.findStockList();
        for (Stock stock : stockList) {
            StockListVO stockListVO = dbStockService.getStockListVO(stock.getStockGid());
            Date nowDate = new Date();
            String ymd_date = DateTimeUtil.dateToStr(nowDate, "yyyy-MM-dd");
            String hm_date = DateTimeUtil.dateToStr(nowDate, "HH:mm");
            StockMarketsDay stockMarketsDay = new StockMarketsDay();
            stockMarketsDay.setStockId(stock.getId());
            stockMarketsDay.setStockName(stock.getStockName());
            stockMarketsDay.setStockCode(stock.getStockCode());
            stockMarketsDay.setStockGid(stock.getStockGid());
            stockMarketsDay.setYmd(ymd_date);
            stockMarketsDay.setHms(hm_date);
            stockMarketsDay.setNowPrice(new BigDecimal(stockListVO.getNowPrice()));
            stockMarketsDay.setCreaseRate(new BigDecimal(stockListVO.getHcrate().toString()));
            stockMarketsDay.setOpenPx(stockListVO.getOpen_px());
            stockMarketsDay.setClosePx(stockListVO.getPreclose_px());
            stockMarketsDay.setBusinessBalance(stockListVO.getBusiness_balance());
            stockMarketsDay.setBusinessAmount(stockListVO.getBusiness_amount());
            stockMarketsDay.setAddTime(nowDate);
            stockMarketsDay.setAddTimeStr(DateTimeUtil.dateToStr(nowDate));
            int insertCount = this.stockMarketsDayMapper.insert(stockMarketsDay);
            if (insertCount > 0) {
                continue;
            }
            log.error("保存货币 {} 失败， 时间 = {}", stock.getStockName(), ymd_date);
        }

    }


    public BigDecimal selectRateByDaysAndStockCode(Integer stockId, int days) {
        return this.stockMarketsDayMapper.selectRateByDaysAndStockCode(stockId, Integer.valueOf(days));
    }

}

