package com.xc.service.impl;


import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xc.common.ServerResponse;
import com.xc.dao.AgentUserMapper;
import com.xc.dao.UserCashDetailMapper;
import com.xc.pojo.AgentUser;
import com.xc.pojo.User;
import com.xc.pojo.UserCapitalDetail;
import com.xc.pojo.UserCashDetail;
import com.xc.service.IAgentUserService;
import com.xc.service.IUserCashDetailService;
import com.xc.service.IUserService;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("iUserCashDetailService")
public class UserCashDetailServiceImpl implements IUserCashDetailService {

    @Autowired
    UserCashDetailMapper userCashDetailMapper;

    @Autowired
    IUserService iUserService;

    @Autowired
    IAgentUserService iAgentUserService;

    @Autowired
    AgentUserMapper agentUserMapper;

    public ServerResponse<PageInfo> findUserCashDetailList(Integer positionId, HttpServletRequest request, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        User user = this.iUserService.getCurrentUser(request);
        List<UserCashDetail> userCashDetails = this.userCashDetailMapper.findUserCashDetailList(user.getId(), positionId);
        PageInfo pageInfo = new PageInfo(userCashDetails);
        return ServerResponse.createBySuccess(pageInfo);
    }

    public ServerResponse<PageInfo> findUserCapitalDetailList(Integer positionId, HttpServletRequest request, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        User user = this.iUserService.getCurrentUser(request);
        List<UserCapitalDetail> userCashDetails = this.userCashDetailMapper.findUserCapitalDetailList(user.getId(), null,null);
        PageInfo pageInfo = new PageInfo(userCashDetails);
        return ServerResponse.createBySuccess(pageInfo);
    }

    public ServerResponse<PageInfo> listByAgent(Integer userId, String userName, Integer agentId, Integer positionId, HttpServletRequest request, int pageNum, int pageSize) {
        AgentUser currentAgent = this.iAgentUserService.getCurrentAgent(request);
        if (agentId != null) {
            AgentUser agentUser = this.agentUserMapper.selectByPrimaryKey(agentId);
            if (agentUser.getParentId() != currentAgent.getId()) {
                return ServerResponse.createByErrorMsg("不能查询非下级代理用户持仓");
            }
        }
        Integer searchId = null;
        if (agentId == null) {
            searchId = currentAgent.getId();
        } else {
            searchId = agentId;
        }
        PageHelper.startPage(pageNum, pageSize);
        List<UserCapitalDetail> cashDetails = this.userCashDetailMapper.findUserCapitalDetailList(userId, userName, searchId);
        PageInfo pageInfo = new PageInfo(cashDetails);
        return ServerResponse.createBySuccess(pageInfo);
    }

    public ServerResponse<PageInfo> listByAdmin(Integer userId, String userName, Integer agentId, Integer positionId, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<UserCapitalDetail> cashDetails = this.userCashDetailMapper.findUserCapitalDetailList(userId, userName, agentId);
        PageInfo pageInfo = new PageInfo(cashDetails);
        return ServerResponse.createBySuccess(pageInfo);
    }


    public int deleteByUserId(Integer userId) {
        return this.userCashDetailMapper.deleteByUserId(userId);
    }

}
