package com.xc.service.impl;

import com.xc.dao.*;
import com.xc.pojo.*;
import com.xc.service.*;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.xc.common.ServerResponse;
import com.xc.service.*;
import com.xc.utils.DateTimeUtil;
import com.xc.utils.KeyUtils;
import com.xc.utils.stock.BuyAndSellUtils;
import com.xc.utils.stock.GeneratePosition;
import com.xc.utils.stock.GetStayDays;
import com.xc.utils.stock.sina.SinaStockApi;
import com.xc.utils.thread.ThreadUtil;
import com.xc.vo.agent.AgentIncomeVO;
import com.xc.vo.agent.AgentUserListVO;
import com.xc.vo.indexposition.IndexPositionVO;
import com.xc.vo.position.AdminPositionVO;
import com.xc.vo.position.AgentPositionVO;
import com.xc.vo.position.PositionProfitVO;
import com.xc.vo.position.PositionVO;
import com.xc.vo.position.UserPositionVO;
import com.xc.vo.stock.StockListVO;
import com.xc.dao.*;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;


import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Timestamp;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.util.*;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

@Service("iUserPositionService")
public class UserPositionServiceImpl implements IUserPositionService {

    private static final Logger log = LoggerFactory.getLogger(UserPositionServiceImpl.class);
    @Resource
    IUserPositionService iUserPositionService;
    @Autowired
    UserPositionMapper userPositionMapper;
    @Autowired
    IUserService iUserService;
    @Autowired
    ISiteSettingService iSiteSettingService;
    @Autowired
    IStockService iStockService;
    @Autowired
    UserMapper userMapper;
    @Autowired
    UserCashDetailMapper userCashDetailMapper;
    @Autowired
    IAgentUserService iAgentUserService;
    @Autowired
    AgentUserMapper agentUserMapper;
    @Autowired
    SiteTaskLogMapper siteTaskLogMapper;
    @Autowired
    StockMapper stockMapper;

    @Autowired
    DbStockService dbStockService;

    /**
     * 判断当前货币当前价格可成交挂单
     *
     * @return
     */
    @Override
    @Transactional
    public boolean fulfill() {
        List<UserPosition> userPositions = this.userPositionMapper.listByCodeAndState(null, 2, null);
        log.info("挂单=" + userPositions.size());
        if (userPositions.size() > 0) {
            List<IndexInfo> indexInfoList = stockMapper.selectStockList();
            Map<String, BigDecimal> indexMap = new HashMap<>();
            Map<String, BigDecimal> spreadMap = new HashMap<>();
            BigDecimal nowPrice;
            for (IndexInfo indexInfo : indexInfoList) {
                try {
                    //每支code获取现价
                    StockListVO stockListVO = dbStockService.getStockListVO(indexInfo.getCode());
                    stockListVO.setId(Integer.parseInt(indexInfo.getId()));
                    nowPrice = new BigDecimal(stockListVO.getNowPrice());
                    indexMap.put(indexInfo.getCnName(), nowPrice);
                    spreadMap.put(indexInfo.getCnName(), indexInfo.getSpread());
                } catch (Exception e) {
                    // 数据库不存在
                    log.info("数据库不存在");
                }
            }
            for (UserPosition userPosition : userPositions) {
                nowPrice = indexMap.get(userPosition.getStockName());
                BigDecimal spread = spreadMap.get(userPosition.getStockName());
                if ("做多".equals(userPosition.getOrderDirection())) {
                    // 现价小于等于委托价
                    if (nowPrice.compareTo(userPosition.getBuyOrderPrice()) < 1) {
                        nowPrice = userPosition.getBuyOrderPrice().add(spread);
                        toFulfill(userPosition, nowPrice);
                    }
                } else {
                    // 现价大于等于委托价
                    if (nowPrice.compareTo(userPosition.getBuyOrderPrice()) > -1) {
                        nowPrice = userPosition.getBuyOrderPrice().subtract(spread);
                        toFulfill(userPosition, nowPrice);
                    }
                }
            }
        }
        return true;
    }

    private void toFulfill(UserPosition userPosition, BigDecimal buyOrderPrice) {
        User user = userMapper.findById(userPosition.getUserId());
        userPositionMapper.updatePositionStatus(userPosition.getId(), buyOrderPrice, buyOrderPrice.multiply(BigDecimal.valueOf(Double.valueOf(userPosition.getOrderNum()))));
        ThreadUtil.executeInThread(obj -> {
            UserCapitalDetail userCapitalDetail = new UserCapitalDetail();
            userCapitalDetail.setAgentId(user.getAgentId());
            userCapitalDetail.setAgentName(user.getAgentName());
            userCapitalDetail.setUserId(user.getId());
            userCapitalDetail.setUserName(user.getRealName());
            userCapitalDetail.setPositionId(userPosition.getId());
            userCapitalDetail.setDeType(1);
            userCapitalDetail.setIsRead(Integer.valueOf(0));
            userCapitalDetail.setOrderNum(userPosition.getOrderNum());
            userCapitalDetail.setBuyFeeAmt(userPosition.getOrderFee());
            userCapitalDetail.setOrderSpread(userPosition.getOrderSpread());
            userCapitalDetail.setOrderStayFee(new BigDecimal(0));
            userCapitalDetail.setAllProfit(new BigDecimal(0));
            userCapitalDetail.setStockName(userPosition.getStockName());
            userCapitalDetail.setStockCode(userPosition.getStockCode());
            int insertSxfCount = this.userCashDetailMapper.insertUserCapitalDetail(userCapitalDetail);
            if (insertSxfCount > 0) {
                log.info("【用户挂单买入】保存明细记录成功");
            } else {
                log.error("用户挂单买入】保存明细记录出错");
            }
        });
    }

    @Override
    public ServerResponse stopProfitTarget(int id, BigDecimal targetprofit, BigDecimal stoploss, HttpServletRequest request) {
        // 获取最新价格
        UserPosition userPosition = this.userPositionMapper.findPositionById(id);
        if (userPosition == null) {
            return ServerResponse.createByErrorMsg("平仓失败，订单不存在");
        }
        // 获取最近报价 要去小明的表中获取
//        StockListVO stockListVO = SinaStockApi.assembleStockListVO(SinaStockApi.getSinaStock(userPosition.getStockGid()));
//        BigDecimal now_price = new BigDecimal(stockListVO.getNowPrice());
        userPositionMapper.updateStopProfitTarget(id, targetprofit, stoploss);
        // 同时异步去触发一次定时任务
        ThreadUtil.executeInThread(obj -> {
            stopProfitTargetTask(id);
        });
        return ServerResponse.createBySuccess();
    }

    @Override
    @Transactional
    public ServerResponse closeOut(int id, Integer orderNum) throws Exception {
        log.info("【用户交易平仓】 positionSn = {}", id);
        UserPosition userPosition = this.userPositionMapper.findPositionById(id);
        if (userPosition == null) {
            return ServerResponse.createByErrorMsg("平仓失败，订单不存在");
        }
        // 获取交易时间和交易手续费
        ServerResponse stock_res = this.iStockService.findStockByName(userPosition.getStockName());
        if (!stock_res.isSuccess()) {
            return ServerResponse.createByErrorMsg("下单失败，代码错误");
        }
        Stock stock = (Stock) stock_res.getData();
        if (stock == null) {
            log.error("下单出错，网站设置表不存在");
            return ServerResponse.createByErrorMsg("下单失败，系统设置错误");
        }
        if (stock.getIsLock() != 0) {
            return ServerResponse.createByErrorMsg("下单失败，当前货币不能交易");
        }
//        if (!iStockService.isTransState(stock)) {
//            return ServerResponse.createByErrorMsg("平仓失败，不在交易时段内");
//        }
        User user = this.userMapper.selectByPrimaryKey(userPosition.getUserId());
        if (user.getIsLock() == 1) {
            return ServerResponse.createByErrorMsg("平仓失败，用户已被锁定");
        }
        if (userPosition.getSellOrderId() != null) {
            return ServerResponse.createByErrorMsg("平仓失败，此订单已平仓");
        }
        if (1 == userPosition.getIsLock()) {
            return ServerResponse.createByErrorMsg("平仓失败 " + userPosition.getLockMsg());
        }
        //每支股票code获取现价
        StockListVO stockListVO = dbStockService.getStockListVO(userPosition.getStockGid());
        BigDecimal now_price = new BigDecimal(stockListVO.getNowPrice());
        if (now_price.compareTo(new BigDecimal("0")) != 1) {
            log.error("货币 = {} 收到报价 = {}", userPosition.getStockName(), now_price);
            return ServerResponse.createByErrorMsg("报价0，平仓失败，请稍后再试");
        }
        // 实际手数
        boolean isAll = false;
        Integer buyNumAll = userPosition.getOrderNum();
        Integer buyNum;
        if (null != orderNum && orderNum > buyNumAll) {
            return ServerResponse.createByErrorMsg("卖出数量大于购买数量");
        } else if (null == orderNum || orderNum == buyNumAll) {
            buyNum = buyNumAll;
            isAll = true;
        } else {
            buyNum = orderNum;
        }
        // 购买所使用金额
        BigDecimal all_buy_amt = userPosition.getBuyOrderPrice().multiply(new BigDecimal(buyNum));
        // 现价购买所需的金额
        BigDecimal all_sell_amt = now_price.multiply(new BigDecimal(buyNum));
        // 计算盈亏
        BigDecimal profitLoss = new BigDecimal("0");
        if ("做多".equals(userPosition.getOrderDirection())) {
            log.info("买卖方向：{}", "涨");
            profitLoss = all_sell_amt.subtract(all_buy_amt);
        } else {
            log.info("买卖方向：{}", "跌");
            profitLoss = all_buy_amt.subtract(all_sell_amt);
        }
        log.info("买入总金额 = {} , 卖出总金额 = {} , 盈亏 = {}", all_buy_amt, all_sell_amt, profitLoss);
        BigDecimal user_all_amt = user.getUserAmt();
        BigDecimal user_enable_amt = user.getEnableAmt();
        log.info("用户原本总资金 = {} , 可用 = {}", user_all_amt, user_enable_amt);
        BigDecimal buy_fee_amt = userPosition.getOrderFee();
        log.info("买入手续费 = {}", buy_fee_amt);
        BigDecimal orderStayFee = userPosition.getOrderStayFee();
        log.info("留仓费 = {}", orderStayFee);
        BigDecimal all_fee_amt = buy_fee_amt.add(orderStayFee);
        log.info("总的手续费费用 = {}", all_fee_amt);
        // 设置卖时的订单信息和价格
        userPosition.setSellOrderId(GeneratePosition.getPositionId());
        userPosition.setSellOrderPrice(now_price);
        userPosition.setSellOrderTime(new Date());
        profitLoss = profitLoss.multiply(new BigDecimal(stock.getContractSize()));
        BigDecimal all_profit = profitLoss;
        userPosition.setProfitAndLose(all_profit);
        userPosition.setAllProfitAndLose(all_profit);
        userPosition.setOrderNum(buyNum);
        userPosition.setOrderTotalPrice(userPosition.getBuyOrderPrice().multiply(new BigDecimal(buyNum)));
        // 用原本的手续费计算出单笔的手续费 再得出总体的手续费
        userPosition.setOrderFee(userPosition.getOrderFee().divide(new BigDecimal(buyNumAll), 4, 4).multiply(new BigDecimal(buyNum)));
        //冻结保证金
        userPosition.setDepositFreezeAmt(userPosition.getDepositFreezeAmt().divide(new BigDecimal(buyNumAll), 4, 4).multiply(new BigDecimal(buyNum)));
        int updatePositionCount = this.userPositionMapper.updateByPrimaryKeySelective(userPosition);
        if (!isAll) {
            UserPosition userPositionInsert = new UserPosition();
            userPositionInsert.setPositionType(userPosition.getPositionType());
            userPositionInsert.setPositionSn(KeyUtils.getUniqueKey());
            userPositionInsert.setUserId(userPosition.getUserId());
            userPositionInsert.setNickName(userPosition.getNickName());
            userPositionInsert.setAgentId(userPosition.getAgentId());
            userPositionInsert.setAgentName(userPosition.getAgentName());
            userPositionInsert.setStockCode(userPosition.getStockCode());
            userPositionInsert.setStockName(userPosition.getStockName());
            userPositionInsert.setStockGid(userPosition.getStockGid());
            userPositionInsert.setStockSpell(userPosition.getStockSpell());
            userPositionInsert.setBuyOrderId(GeneratePosition.getPositionId());
            userPositionInsert.setBuyOrderTime(new Date());
            userPositionInsert.setOrderDirection(userPosition.getOrderDirection());
            userPositionInsert.setStockPlate(userPosition.getStockPlate());
            userPositionInsert.setIsLock(userPosition.getIsLock());
            userPositionInsert.setOrderLever(userPosition.getOrderLever());
            Integer num = buyNumAll - buyNum;
            userPositionInsert.setOrderNum(num);
            userPositionInsert.setBuyOrderPrice(userPosition.getBuyOrderPrice());
            userPositionInsert.setOrderTotalPrice(userPosition.getBuyOrderPrice().multiply(new BigDecimal(num)));
            // 用原本的手续费计算出单笔的手续费 再得出总体的手续费
            userPositionInsert.setOrderFee(userPosition.getOrderFee().divide(new BigDecimal(buyNum), 4, 4).multiply(new BigDecimal(num)));
            userPositionInsert.setProfitAndLose(BigDecimal.ZERO);
            userPositionInsert.setOrderStayDays(0);
            userPositionInsert.setOrderStayFee(BigDecimal.ZERO);
            userPositionInsert.setOrderSpread(BigDecimal.ZERO);
            //冻结保证金
            userPositionInsert.setDepositFreezeAmt(userPosition.getDepositFreezeAmt().divide(new BigDecimal(buyNum), 4, 4).multiply(new BigDecimal(num)));
            //是否挂单
            userPositionInsert.setEntryOrders(0);
            this.userPositionMapper.insertSelective(userPositionInsert);
        }
        if (updatePositionCount > 0) {
            log.info("【用户平仓】修改浮动盈亏记录成功");
        } else {
            log.error("用户平仓】修改浮动盈亏记录出错");
            throw new Exception("用户平仓】修改浮动盈亏记录出错");
        }
        //修改持仓表记录
//        BigDecimal freez_amt = all_buy_amt;
//        BigDecimal reckon_all = user_all_amt.add(all_profit);
//        BigDecimal reckon_enable = user_enable_amt.add(all_profit).add(freez_amt);
        BigDecimal reckon_all = user.getUserAmt().add(profitLoss);
        BigDecimal reckon_enable = user.getEnableAmt().add(userPosition.getDepositFreezeAmt()).add(profitLoss);
        log.info("用户平仓后的总资金  = {} , 可用资金 = {}", reckon_all, reckon_enable);
        user.setUserAmt(reckon_all);
        user.setEnableAmt(reckon_enable);
        int updateUserCount = this.userMapper.updateByPrimaryKeySelective(user);
        if (updateUserCount > 0) {
            log.info("【用户平仓】修改用户金额成功");
        } else {
            log.error("用户平仓】修改用户金额出错");
            throw new Exception("用户平仓】修改用户金额出错");
        }
        UserCapitalDetail userCapitalDetail = new UserCapitalDetail();
        userCapitalDetail.setAgentId(user.getAgentId());
        userCapitalDetail.setAgentName(user.getAgentName());
        userCapitalDetail.setUserId(user.getId());
        userCapitalDetail.setUserName(user.getRealName());
        userCapitalDetail.setPositionId(userPosition.getId());
        userCapitalDetail.setDeType(2);
        userCapitalDetail.setIsRead(0);
        userCapitalDetail.setOrderNum(buyNum);
        userCapitalDetail.setBuyFeeAmt(buy_fee_amt);
        userCapitalDetail.setOrderSpread(new BigDecimal(0));
        userCapitalDetail.setOrderStayFee(orderStayFee);
        userCapitalDetail.setAllProfit(all_profit);
        userCapitalDetail.setStockName(userPosition.getStockName());
        userCapitalDetail.setStockCode(userPosition.getStockCode());
        int insertSxfCount = this.userCashDetailMapper.insertUserCapitalDetail(userCapitalDetail);
        if (insertSxfCount > 0) {
            log.info("【用户平仓】保存明细记录成功");
        } else {
            log.error("用户平仓】保存明细记录出错");
            throw new Exception("用户平仓】保存明细记录出错");
        }
        return ServerResponse.createBySuccessMsg("平仓成功！");
    }

    @Override
    public ServerResponse closeOutList(List<Integer> list) throws Exception {
        for (Integer userId : list) {
            iUserPositionService.closeOut(userId, null);
        }
        return ServerResponse.createBySuccessMsg("平仓成功！");
    }

    @Override
    public ServerResponse oneKeyCloseOut(HttpServletRequest request) {
        User user = this.iUserService.getCurrentRefreshUser(request);
        if (iUserService.ForceSellByUserId(user.getId())) {
            return ServerResponse.createBySuccessMsg("一键平仓成功！");
        }
        return ServerResponse.createByErrorMsg("平仓失败！！！");
    }

    private void toStopProfitTargetById(int id) throws Exception {
        // 获取货币当前价格
        List<IndexInfo> indexInfoList = stockMapper.selectStockList();
        Map<String, BigDecimal> indexMap = new HashMap<>();
        BigDecimal nowPrice;
        for (IndexInfo indexInfo : indexInfoList) {
            if (StringUtils.isNotEmpty(indexInfo.getDataDbName())) {
                try {
                    //每支code获取现价
                    StockListVO stockListVO = dbStockService.getStockListVO(indexInfo.getCode());
                    nowPrice = new BigDecimal(stockListVO.getNowPrice());
                    indexMap.put(indexInfo.getCnName(), nowPrice);
                } catch (Exception e) {
                    // 数据库不存在
                    log.info("数据库不存在");
                }
            }
        }
        // 获取有设置止盈的账号
        UserPosition userPosition = userPositionMapper.selectByPrimaryKey(id);
        nowPrice = indexMap.get(userPosition.getStockName());
        if (judgeIsCloseOut(userPosition, nowPrice)) {
            closeOut(userPosition.getId(), null);
        }
    }

    @Override
    public void toStopProfitTarget() throws Exception {
        // 获取货币当前价格
        List<IndexInfo> indexInfoList = stockMapper.selectStockList();
        Map<String, BigDecimal> indexMap = new HashMap<>();
        BigDecimal nowPrice;
        for (IndexInfo indexInfo : indexInfoList) {
            if (StringUtils.isNotEmpty(indexInfo.getDataDbName())) {
                try {
                    //每支code获取现价
                    StockListVO stockListVO = dbStockService.getStockListVO(indexInfo.getCode());
                    nowPrice = new BigDecimal(stockListVO.getNowPrice());
                    indexMap.put(indexInfo.getCnName(), nowPrice);
                } catch (Exception e) {
                    // 数据库不存在
                    log.info("数据库不存在");
                }
            }
        }
        // 获取有设置止盈的账号
        List<UserPosition> userPositionList = userPositionMapper.selectPositionByStopProfitTarget();
        for (UserPosition userPosition : userPositionList) {
            nowPrice = indexMap.get(userPosition.getStockName());
            if (judgeIsCloseOut(userPosition, nowPrice)) {
                closeOut(userPosition.getId(), null);
            }
        }
    }

    private boolean judgeIsCloseOut(UserPosition userPosition, BigDecimal nowPrice) {
        //判断做多 做空
        if ("做多".equals(userPosition.getOrderDirection())) {
            if (!(userPosition.getTargetprofit().compareTo(BigDecimal.ZERO) == 0) && nowPrice.compareTo(userPosition.getTargetprofit()) > -1) {
                //  当前价格 大于等于 止盈价格 进行平仓操作
                return true;
            } else if (!(userPosition.getStoploss().compareTo(BigDecimal.ZERO) == 0) && nowPrice.compareTo(userPosition.getStoploss()) < 1) {
                // 当前价格 小于等于 止损价格 进行平仓操作
                return true;
            }
        } else {
            if (!(userPosition.getStoploss().compareTo(BigDecimal.ZERO) == 0) && nowPrice.compareTo(userPosition.getStoploss()) > -1) {
                // 当前价格 大于等于 止损价格 进行平仓操作
                return true;
            } else if (!(userPosition.getTargetprofit().compareTo(BigDecimal.ZERO) == 0) && nowPrice.compareTo(userPosition.getTargetprofit()) < 1) {
                // 当前价格  小于等于 止盈价格 进行平仓操作
                return true;
            }
        }
        return false;
    }

    @Override
    public void calcDemurrage() {
        // 获取所有正在持仓的用户ID
        List<Integer> userIdList = userPositionMapper.selectPositionUserId();
        // 获取各货币过夜费
        List<IndexInfo> indexInfoList = stockMapper.selectStockList();
        Map<String, Integer> indexMap = new HashMap<>();
        for (IndexInfo indexInfo : indexInfoList) {
            indexMap.put(indexInfo.getCnName(), indexInfo.getDemurrage());
        }
        for (Integer userId : userIdList) {
            // 获取用户余额
            User user = userMapper.selectByPrimaryKey(userId);
            BigDecimal enableAmt = user.getEnableAmt();
            BigDecimal allAmt = user.getUserAmt();
            // 当前用户需要的总过夜费
            int demurrageSum = userPositionMapper.selectUserDemurrageSum(userId);
            if (-1 == enableAmt.compareTo(new BigDecimal(demurrageSum))) {
                // 用户余额小于需要扣除的过夜费
                // 平仓操作
                iUserService.ForceSellByUserId(userId);
                continue;
            }
            // 获取用户持仓记录
            List<UserPosition> userPositionList = userPositionMapper.selectPositionByUserId(userId);
            // 计算过夜费修改数据
            BigDecimal orderStayFee;
            for (UserPosition userPosition : userPositionList) {
                orderStayFee = new BigDecimal(indexMap.get(userPosition.getStockName())).multiply(BigDecimal.valueOf(userPosition.getOrderNum()));
                userPosition.setOrderStayFee(userPosition.getOrderStayFee().add(orderStayFee));
                // 修改持仓表过夜费
                userPositionMapper.updateOrderStayFee(userPosition);
            }
            // 存库
            enableAmt = enableAmt.subtract(new BigDecimal(demurrageSum));
            allAmt = allAmt.subtract(new BigDecimal(demurrageSum));
            // 修改用户余额
            user.setEnableAmt(enableAmt);
            user.setUserAmt(allAmt);
            int updateUserCount = this.userMapper.updateByPrimaryKeySelective(user);
            if (updateUserCount > 0) {
                log.info("【过夜费】修改用户金额成功");
            } else {
                log.error("过夜费】修改用户金额出错");
            }
        }
    }

    @Override
    @Transactional
    public ServerResponse revoke(Integer positionId, HttpServletRequest request) throws Exception {
        try {
            User user = this.iUserService.getCurrentRefreshUser(request);
            if (null == user || null == user.getId()) {
                return ServerResponse.createByErrorMsg("用户未登录！");
            }
            if (null == positionId || positionId < 0) {
                return ServerResponse.createByErrorMsg("挂单号错误！");
            }
            UserPosition userPosition = this.userPositionMapper.selectByPrimaryKeyAndStatus(positionId);
            if (null == userPosition || userPosition.getDepositFreezeAmt() == null) {
                return ServerResponse.createByErrorMsg("挂单不存在！");
            }
            user.setEnableAmt(user.getEnableAmt().add(userPosition.getDepositFreezeAmt()).add(userPosition.getOrderFee()));
            log.info("冻结保证金 = " + userPosition.getDepositFreezeAmt());
            log.info("可用余额 = " + user.getEnableAmt());
            //        user.setUserAmt(user.getUserAmt().add(userPosition.getDepositFreezeAmt()));
            //        log.info("总资金 = "+user.getUserAmt());
            user.setUserAmt(user.getUserAmt().add(userPosition.getOrderFee()));
            int updateUserCount = this.userMapper.updateByPrimaryKeySelective(user);
            if (updateUserCount > 0) {
                log.info("【用户撤单】修改用户金额成功");
            } else {
                log.error("用户撤单】修改用户金额出错");
                throw new Exception("用户撤单】修改用户金额出错");
            }
            int res = this.userPositionMapper.deleteByPrimaryKey(positionId);
            if (res > 0) {
                return ServerResponse.createBySuccess("撤单成功");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.toString());
        }
        return ServerResponse.createByErrorMsg("撤单失败");
    }

    // 挂单
    @Override
    @Transactional
    public ServerResponse buyEntryOrders(Integer stockId, Integer buyNum, Integer buyType, Integer entryOrders, BigDecimal buyOrderPrice, HttpServletRequest request) throws Exception {
        try {
            User user = this.iUserService.getCurrentRefreshUser(request);
            if (StringUtils.isBlank(user.getRealName()) || StringUtils.isBlank(user.getIdCard())) {
                return ServerResponse.createByErrorMsg("挂单失败，请先实名认证");
            }
            if (user.getIsLock() == 1) {
                return ServerResponse.createByErrorMsg("挂单失败，账户已被锁定");
            }
            Stock stock = null;
            ServerResponse stock_res = this.iStockService.findStockById(stockId);
            if (!stock_res.isSuccess()) {
                return ServerResponse.createByErrorMsg("挂单失败，货币代码错误");
            }
            stock = (Stock) stock_res.getData();
            if (stock == null) {
                log.error("挂单出错，网站设置表不存在");
                return ServerResponse.createByErrorMsg("挂单失败，系统设置错误");
            }
            if (stock.getIsLock() != 0) {
                return ServerResponse.createByErrorMsg("挂单失败，当前货币不能交易");
            }
            if (buyNum < stock.getBuyMinNum()) {
                return ServerResponse.createByErrorMsg("挂单失败，购买数量小于" + stock.getBuyMinNum() + "手");
            }
            if (buyNum > stock.getBuyMaxNum()) {
                return ServerResponse.createByErrorMsg("挂单失败，购买数量大于" + stock.getBuyMaxNum() + "手");
            }
            Integer userPositionAmt = findUserPositionAmt(user.getId(), stock.getStockCode());
            if ((userPositionAmt + buyNum) > stock.getPositionsMax()) {
                return ServerResponse.createByErrorMsg("挂单失败，本次挂单手数：" + buyNum + "，持仓手数大于" + stock.getPositionsMax() + "手，目前持仓手数：" + userPositionAmt);
            }
            //每支code获取现价
            StockListVO stockListVO = dbStockService.getStockListVO(stock.getStockGid());
            BigDecimal now_price = new BigDecimal(stockListVO.getNowPrice());
            if (now_price.compareTo(new BigDecimal("0")) == 0) {
                return ServerResponse.createByErrorMsg("报价0，请稍后再试");
            }
            log.info("挂单现价=" + now_price);
            log.info("挂单价=" + buyOrderPrice);
            log.info("价差=" + stock.getEntryOrdersAgio());
            BigDecimal addNowprice = new BigDecimal(0);
//            BigDecimal subNowprice = new BigDecimal(0);
            addNowprice = now_price.add(stock.getEntryOrdersAgio());
//            subNowprice = now_price.subtract(stock.getEntryOrdersAgio());
//            log.info("挂单价差做涨下线="+now_price);
//            if(!(buyOrderPrice.compareTo(addNowprice)  > -1 || buyOrderPrice.compareTo(subNowprice) < 1)){
//                return ServerResponse.createByErrorMsg("挂单价格不在价差范围内");
//            }
            if (buyType == 0) {
                // 做多
                // 委托价 大于 现价+价格差 才可以设置
                if (buyOrderPrice.compareTo(addNowprice) == 1) {
                    // 不可以设置
                    return ServerResponse.createByErrorMsg("挂单价格不在价差范围内");
                }
            } else {
                // 做空
                // 委托价 小于 现价+价格差 才可以设置
                if (buyOrderPrice.compareTo(addNowprice) == -1) {
                    return ServerResponse.createByErrorMsg("挂单价格不在价差范围内");
                }
            }
            // 用户可用资金
            BigDecimal user_enable_amt = user.getEnableAmt();
            // 委托价 * 购买手数
            BigDecimal buy_amt_autual = buyOrderPrice.multiply(new BigDecimal(buyNum));
            log.info("用户可用金额 = {}  实际购买金额 =  {}", user_enable_amt, buy_amt_autual);
            // 手续费
            BigDecimal position_freez = buyOrderPrice.multiply(new BigDecimal(buyNum)).multiply(stock.getTransFee()).multiply(new BigDecimal(stock.getContractSize()));
            //冻结保证金
            BigDecimal depositFreezeAmt = buyOrderPrice.multiply(new BigDecimal(stock.getContractSize())).multiply(new BigDecimal(buyNum)).divide(new BigDecimal(stock.getSiteLever()), 2, 4);
            log.info("冻结保证金 = " + depositFreezeAmt);
            BigDecimal subtractDepositFreezeAmt = user_enable_amt.subtract(depositFreezeAmt).subtract(position_freez);
            log.info("挂单冻结保证金=" + depositFreezeAmt);
//            depositFreezeAmt = depositFreezeAmt.add(position_freez);
            log.info("挂单冻结手续费=" + position_freez);
            log.info("挂单冻结金额=" + depositFreezeAmt);
            // 保证金和手续费 大于 用户可用资金 挂单失败
            if ((depositFreezeAmt.add(position_freez)).compareTo(user.getEnableAmt()) > -1) {
                log.info("用户余额不足冻结保证金");
                return ServerResponse.createByErrorMsg("挂单失败，用户余额不足冻结保证金：" + depositFreezeAmt);
            }
            user.setUserAmt(user.getUserAmt().subtract(position_freez));
            user.setEnableAmt(subtractDepositFreezeAmt);
            int updateUserCount = this.userMapper.updateByPrimaryKeySelective(user);
            if (updateUserCount > 0) {
                log.info("【用户交易挂单】修改用户金额成功");
            } else {
                log.error("用户交易挂单】修改用户金额出错");
                throw new Exception("用户交易挂单】修改用户金额出错");
            }
            UserPosition userPosition = new UserPosition();
            userPosition.setPositionType(user.getAccountType());
            userPosition.setPositionSn(KeyUtils.getUniqueKey());
            userPosition.setUserId(user.getId());
            userPosition.setNickName(user.getRealName());
            userPosition.setAgentId(user.getAgentId());
            userPosition.setAgentName(user.getAgentName());
            userPosition.setStockCode(stock.getStockCode());
            userPosition.setStockName(stock.getStockName());
            userPosition.setStockGid(stock.getStockGid());
            userPosition.setStockSpell(stock.getStockSpell());
            userPosition.setBuyOrderId(GeneratePosition.getPositionId());
            userPosition.setBuyOrderTime(new Date());
            userPosition.setBuyOrderPrice(buyOrderPrice);
            userPosition.setOrderDirection((buyType == 0) ? "做多" : "做空");
            userPosition.setOrderNum(buyNum);
            if (stock.getStockPlate() != null) {
                userPosition.setStockPlate(stock.getStockPlate());
            }
            userPosition.setIsLock(0);
            userPosition.setOrderLever(new BigDecimal(0));
            userPosition.setOrderTotalPrice(buy_amt_autual);
            userPosition.setOrderFee(position_freez);
            BigDecimal profit_and_lose = new BigDecimal("0");
            userPosition.setProfitAndLose(profit_and_lose);
            userPosition.setOrderStayDays(0);
            userPosition.setOrderStayFee(new BigDecimal("0"));
            userPosition.setOrderSpread(stock.getSpread().multiply(new BigDecimal(buyNum)));
            userPosition.setDepositFreezeAmt(depositFreezeAmt);
            userPosition.setEntryOrders(1);
            int insertPositionCount = this.userPositionMapper.insertSelective(userPosition);
            if (insertPositionCount > 0) {
                log.info("【用户交易挂单】保存持仓记录成功");
            } else {
                log.error("用户交易挂单】保存持仓记录出错");
                throw new Exception("用户交易挂单】保存持仓记录出错");
            }
        } catch (Exception e) {
            log.error(e.toString());
            e.printStackTrace();
            return ServerResponse.createByErrorMsg("挂单失败，请重试！");
        }
        return ServerResponse.createBySuccess("挂单成功");
    }

    @Override
    @Transactional
    public ServerResponse buy(Integer stockId, Integer buyNum, Integer buyType, HttpServletRequest request) throws Exception {
        //判断用户有没有买的资格
        User user = this.iUserService.getCurrentRefreshUser(request);
        if (StringUtils.isBlank(user.getRealName()) || StringUtils.isBlank(user.getIdCard())) {
            return ServerResponse.createByErrorMsg("下单失败，请先实名认证");
        }
        log.info("用户 {} 下单，货币id = {} ，数量 = {} , 方向 = {} ", user
                .getId(), stockId, buyNum, buyType);
        if (user.getIsLock() == 1) {
            return ServerResponse.createByErrorMsg("下单失败，账户已被锁定");
        }
        //判断货币是否能交易
        Stock stock = null;
        ServerResponse stock_res = this.iStockService.findStockById(stockId);
        if (!stock_res.isSuccess()) {
            return ServerResponse.createByErrorMsg("下单失败，货币代码错误");
        }
        stock = (Stock) stock_res.getData();
        if (stock == null) {
            log.error("下单出错，网站设置表不存在");
            return ServerResponse.createByErrorMsg("下单失败，系统设置错误");
        }
        if (stock.getIsLock() != 0) {
            return ServerResponse.createByErrorMsg("下单失败，当前货币不能交易");
        }
        //判断交易时间
//        if (!iStockService.isTransState(stock)) {
//            return ServerResponse.createByErrorMsg("下单失败，不在交易时段内");
//        }
        if (buyNum < stock.getBuyMinNum()) {
            return ServerResponse.createByErrorMsg("下单失败，购买数量小于" + stock.getBuyMinNum() + "手");
        }
        if (buyNum > stock.getBuyMaxNum()) {
            return ServerResponse.createByErrorMsg("下单失败，购买数量大于" + stock.getBuyMaxNum() + "手");
        }
        // 判断用户持仓数
        Integer userPositionAmt = findUserPositionAmt(user.getId(), stock.getStockCode());
        if ((userPositionAmt + buyNum) > (stock.getPositionsMax() == null ? 0 : stock.getPositionsMax())) {
            return ServerResponse.createByErrorMsg("下单失败，本次下单手数：" + buyNum + "，持仓手数大于" + stock.getPositionsMax() + "手，目前持仓手数：" + userPositionAmt);
        }
        // 获取股票最新价格
        StockListVO stockListVO = dbStockService.getStockListVO(stock.getStockGid());
        BigDecimal nowPrice = new BigDecimal(stockListVO.getNowPrice());
        if (nowPrice.compareTo(new BigDecimal("0")) == 0) {
            return ServerResponse.createByErrorMsg("报价0，请稍后再试");
        }
        if (buyType == 0) {
            // 做多
            nowPrice = nowPrice.add(stock.getSpread());
        } else {
            nowPrice = nowPrice.subtract(stock.getSpread());
        }
        log.info("用户 {} 下单，货币id = {} ，数量 = {} , 方向 = {}", user.getId(), stockId, buyNum, buyType);
        // 用户可用金
        BigDecimal user_enable_amt = user.getEnableAmt();
        BigDecimal user_amt = user.getUserAmt();
        // 判断金额是否能够买得起   用 可用金额 > 冻结金额 + 手续费 则标示可以购买
        // 手续费
        BigDecimal position_freez = nowPrice.multiply(new BigDecimal(buyNum)).multiply(stock.getTransFee()).multiply(new BigDecimal(stock.getContractSize()));
        log.info("手续费 = " + position_freez);
        //冻结保证金 = 现价* 合约单位 * 手数 / 杠杆
        BigDecimal depositFreezeAmt = nowPrice.multiply(new BigDecimal(stock.getContractSize())).multiply(new BigDecimal(buyNum)).divide(new BigDecimal(stock.getSiteLever()), 2, 4);
        log.info("冻结保证金 = " + depositFreezeAmt);
        BigDecimal buy_amt = position_freez.add(depositFreezeAmt);
        if (user_enable_amt.compareTo(buy_amt) == -1) {
            return ServerResponse.createByErrorMsg("下单失败，余额可用金额小于" + buy_amt + "元");
        }
        user.setEnableAmt(user_enable_amt.subtract(position_freez).subtract(depositFreezeAmt));
        log.info("可用余额 = " + user_enable_amt.subtract(position_freez).subtract(depositFreezeAmt));
        user.setUserAmt(user_amt.subtract(position_freez));
        int updateUserCount = this.userMapper.updateByPrimaryKeySelective(user);
        if (updateUserCount > 0) {
            log.info("【用户交易下单】修改用户金额成功");
        } else {
            log.error("用户交易下单】修改用户金额出错");
            throw new Exception("用户交易下单】修改用户金额出错");
        }
        Stock finalStock = stock;
        ThreadUtil.executeInThread(obj -> {
//            UserCashDetail ucd = new UserCashDetail();
//            ucd.setPositionId(user.getId());
//            ucd.setAgentId(user.getAgentId());
//            ucd.setAgentName(user.getAgentName());
//            ucd.setUserId(user.getId());
//            ucd.setUserName(user.getRealName());
//            ucd.setDeType("手续费");
//            ucd.setDeAmt(position_freez);
//            ucd.setDeSummary("买入货币，" + finalStock.getStockCode() + "/" + finalStock.getStockName()+",买入手数："+ buyNum + ",手续费：" + position_freez );
//            ucd.setAddTime(new Date());
//            ucd.setIsRead(Integer.valueOf(0));
//            int insertSxfCount = this.userCashDetailMapper.insert(ucd);
            UserCapitalDetail userCapitalDetail = new UserCapitalDetail();
            userCapitalDetail.setAgentId(user.getAgentId());
            userCapitalDetail.setAgentName(user.getAgentName());
            userCapitalDetail.setUserId(user.getId());
            userCapitalDetail.setUserName(user.getRealName());
            userCapitalDetail.setDeType(1);
            userCapitalDetail.setIsRead(Integer.valueOf(0));
            userCapitalDetail.setOrderNum(buyNum);
            userCapitalDetail.setBuyFeeAmt(position_freez);
            userCapitalDetail.setOrderSpread(finalStock.getSpread().multiply(new BigDecimal(buyNum)));
            userCapitalDetail.setOrderStayFee(new BigDecimal(0));
            userCapitalDetail.setAllProfit(new BigDecimal(0));
            userCapitalDetail.setStockName(finalStock.getStockName());
            userCapitalDetail.setStockCode(finalStock.getStockCode());
            int insertSxfCount = this.userCashDetailMapper.insertUserCapitalDetail(userCapitalDetail);
            if (insertSxfCount > 0) {
                log.info("【用户买入】保存明细记录成功");
            } else {
                log.error("用户买入】保存明细记录出错");
            }
        });
        UserPosition userPosition = new UserPosition();
        userPosition.setPositionType(user.getAccountType());
        userPosition.setPositionSn(KeyUtils.getUniqueKey());
        userPosition.setUserId(user.getId());
        userPosition.setNickName(user.getRealName());
        userPosition.setAgentId(user.getAgentId());
        userPosition.setAgentName(user.getAgentName());
        userPosition.setStockCode(stock.getStockCode());
        userPosition.setStockName(stock.getStockName());
        userPosition.setStockGid(stock.getStockGid());
        userPosition.setStockSpell(stock.getStockSpell());
        userPosition.setBuyOrderId(GeneratePosition.getPositionId());
        userPosition.setBuyOrderTime(new Date());
        userPosition.setBuyOrderPrice(nowPrice);
        userPosition.setOrderDirection((buyType == 0) ? "做多" : "做空");
        userPosition.setOrderNum(buyNum);
        if (stock.getStockPlate() != null) {
            userPosition.setStockPlate(stock.getStockPlate());
        }
        userPosition.setIsLock(0);
        userPosition.setOrderLever(new BigDecimal(0));
        userPosition.setOrderTotalPrice(nowPrice.multiply(new BigDecimal(buyNum)));
        // 源计算手续费 手续费直接扣余额 不足提示
//        BigDecimal buy_fee_amt = buy_amt.multiply(new BigDecimal(stock.getTransFee())).setScale(2, 4);
//        log.info("用户购买手续费（配资后总资金 * 百分比） = {}", buy_fee_amt);
        userPosition.setOrderFee(position_freez);
        BigDecimal profit_and_lose = new BigDecimal("0");
        userPosition.setProfitAndLose(profit_and_lose);
        // 源减去手续费  现在改成扣余额
//        BigDecimal all_profit_and_lose = profit_and_lose.subtract(buy_fee_amt).subtract(buy_yhs_amt);
//        userPosition.setAllProfitAndLose(all_profit_and_lose);
        userPosition.setOrderStayDays(0);
        userPosition.setOrderStayFee(new BigDecimal("0"));
        userPosition.setOrderSpread(stock.getSpread().multiply(new BigDecimal(buyNum)));
        log.info("点差" + (stock.getSpread().multiply(new BigDecimal(buyNum))));
        //冻结保证金
        userPosition.setDepositFreezeAmt(depositFreezeAmt);
        //是否挂单
        userPosition.setEntryOrders(0);

        int insertPositionCount = this.userPositionMapper.insertSelective(userPosition);
        if (insertPositionCount > 0) {
            log.info("【用户交易下单】保存持仓记录成功");
        } else {
            log.error("用户交易下单】保存持仓记录出错");
            throw new Exception("用户交易下单】保存持仓记录出错");
        }
        return ServerResponse.createBySuccess("下单成功");
    }

    public ServerResponse sell(String positionSn, int doType) throws Exception {
        log.info("【用户交易平仓】 positionSn = {} ， dotype = {}", positionSn, Integer.valueOf(doType));
        SiteSetting siteSetting = this.iSiteSettingService.getSiteSetting();
        UserPosition userPosition = this.userPositionMapper.findPositionBySn(positionSn);
        if (userPosition == null) {
            return ServerResponse.createByErrorMsg("平仓失败，订单不存在");
        }
        if (siteSetting == null) {
            log.error("平仓出错，网站设置表不存在");
            return ServerResponse.createByErrorMsg("下单失败，系统设置错误");
        }
        if (doType != 0) {
            // 交易时间
            String am_begin = siteSetting.getTransAmBegin();
            String am_end = siteSetting.getTransAmEnd();
            String pm_begin = siteSetting.getTransPmBegin();
            String pm_end = siteSetting.getTransPmEnd();
            boolean am_flag = BuyAndSellUtils.isTransTime(am_begin, am_end);
            boolean pm_flag = BuyAndSellUtils.isTransTime(pm_begin, pm_end);
            log.info("是否在上午交易时间 = {} 是否在下午交易时间 = {}", Boolean.valueOf(am_flag), Boolean.valueOf(pm_flag));
            if (!am_flag && !pm_flag) {
                return ServerResponse.createByErrorMsg("平仓失败，不在交易时段内");
            }
        }
        User user = this.userMapper.selectByPrimaryKey(userPosition.getUserId());
        if (user.getIsLock().intValue() == 1) {
            return ServerResponse.createByErrorMsg("平仓失败，用户已被锁定");
        }
        if (userPosition.getSellOrderId() != null) {
            return ServerResponse.createByErrorMsg("平仓失败，此订单已平仓");
        }
        if (1 == userPosition.getIsLock().intValue()) {
            return ServerResponse.createByErrorMsg("平仓失败 " + userPosition.getLockMsg());
        }
        Stock stock = stockMapper.findStockByName(userPosition.getStockName());
        // 获取股票最新价格
        StockListVO stockListVO = dbStockService.getStockListVO(stock.getStockGid());
        BigDecimal now_price = new BigDecimal(stockListVO.getNowPrice());
        if (now_price.compareTo(new BigDecimal("0")) != 1) {
            log.error("货币 = {} 收到报价 = {}", userPosition.getStockName(), now_price);
            return ServerResponse.createByErrorMsg("报价0，平仓失败，请稍后再试");
        }
        double stock_crease = stockListVO.getHcrate().doubleValue();
        BigDecimal zsPrice = new BigDecimal(stockListVO.getPreclose_px());
        BigDecimal ztPrice = zsPrice.multiply(new BigDecimal("0.1")).add(zsPrice);
        ztPrice = ztPrice.setScale(2, 4);
        BigDecimal chaPrice = ztPrice.subtract(zsPrice);
        BigDecimal ztRate = chaPrice.multiply(new BigDecimal("100")).divide(zsPrice, 2, RoundingMode.HALF_UP);
        ztRate = ztRate.negate();
        log.info("货币当前涨跌幅 = {} 跌停幅度 = {}", Double.valueOf(stock_crease), ztRate);
        if ((new BigDecimal(String.valueOf(stock_crease))).compareTo(ztRate) == 0 && "做多"
                .equals(userPosition.getOrderDirection())) {
            return ServerResponse.createByErrorMsg("当前货币已跌停不能卖出");
        }
        Integer buy_num = userPosition.getOrderNum();
        BigDecimal all_buy_amt = userPosition.getOrderTotalPrice();
        BigDecimal all_sell_amt = now_price.multiply(new BigDecimal(buy_num.intValue()));
        BigDecimal profitLoss = new BigDecimal("0");
        if ("做多".equals(userPosition.getOrderDirection())) {
            log.info("买卖方向：{}", "涨");
            profitLoss = all_sell_amt.subtract(all_buy_amt);
        } else {
            log.info("买卖方向：{}", "跌");
            profitLoss = all_buy_amt.subtract(all_sell_amt);
        }
        log.info("买入总金额 = {} , 卖出总金额 = {} , 盈亏 = {}", new Object[]{all_buy_amt, all_sell_amt, profitLoss});
        BigDecimal user_all_amt = user.getUserAmt();
        BigDecimal user_enable_amt = user.getEnableAmt();
        log.info("用户原本总资金 = {} , 可用 = {}", user_all_amt, user_enable_amt);
        BigDecimal buy_fee_amt = userPosition.getOrderFee();
        log.info("买入手续费 = {}", buy_fee_amt);
        BigDecimal orderSpread = userPosition.getOrderSpread();
        log.info("点差 = {}", orderSpread);
        BigDecimal orderStayFee = userPosition.getOrderStayFee();
        log.info("留仓费 = {}", orderStayFee);
        BigDecimal sell_fee_amt = all_sell_amt.multiply(siteSetting.getSellFee()).setScale(2, 4);
        log.info("卖出手续费 = {}", sell_fee_amt);
        BigDecimal all_fee_amt = buy_fee_amt.add(sell_fee_amt).add(orderSpread).add(orderStayFee);
        log.info("总的手续费费用 = {}", all_fee_amt);
        userPosition.setSellOrderId(GeneratePosition.getPositionId());
        userPosition.setSellOrderPrice(now_price);
        userPosition.setSellOrderTime(new Date());
        BigDecimal order_fee_all = buy_fee_amt.add(sell_fee_amt);
        userPosition.setOrderFee(order_fee_all);
        userPosition.setProfitAndLose(profitLoss);
        BigDecimal all_profit = profitLoss.subtract(all_fee_amt);
        userPosition.setAllProfitAndLose(all_profit);
        int updatePositionCount = this.userPositionMapper.updateByPrimaryKeySelective(userPosition);
        if (updatePositionCount > 0) {
            log.info("【用户平仓】修改浮动盈亏记录成功");
        } else {
            log.error("用户平仓】修改浮动盈亏记录出错");
            throw new Exception("用户平仓】修改浮动盈亏记录出错");
        }
        BigDecimal freez_amt = all_buy_amt.divide(new BigDecimal(userPosition.getOrderLever().intValue()), 2, 4);
        BigDecimal reckon_all = user_all_amt.add(all_profit);
        BigDecimal reckon_enable = user_enable_amt.add(all_profit).add(freez_amt);
        log.info("用户平仓后的总资金  = {} , 可用资金 = {}", reckon_all, reckon_enable);
        user.setUserAmt(reckon_all);
        user.setEnableAmt(reckon_enable);
        int updateUserCount = this.userMapper.updateByPrimaryKeySelective(user);
        if (updateUserCount > 0) {
            log.info("【用户平仓】修改用户金额成功");
        } else {
            log.error("用户平仓】修改用户金额出错");
            throw new Exception("用户平仓】修改用户金额出错");
        }
        UserCapitalDetail userCapitalDetail = new UserCapitalDetail();
        userCapitalDetail.setAgentId(user.getAgentId());
        userCapitalDetail.setAgentName(user.getAgentName());
        userCapitalDetail.setUserId(user.getId());
        userCapitalDetail.setUserName(user.getRealName());
        userCapitalDetail.setPositionId(userPosition.getId());
        userCapitalDetail.setDeType(2);
        userCapitalDetail.setIsRead(Integer.valueOf(0));
        userCapitalDetail.setOrderNum(buy_num);
        userCapitalDetail.setBuyFeeAmt(buy_fee_amt);
        userCapitalDetail.setOrderSpread(orderSpread);
        userCapitalDetail.setOrderStayFee(orderStayFee);
        userCapitalDetail.setAllProfit(all_profit);
        userCapitalDetail.setStockName(userPosition.getStockName());
        userCapitalDetail.setStockCode(userPosition.getStockCode());
        int insertSxfCount = this.userCashDetailMapper.insertUserCapitalDetail(userCapitalDetail);
        if (insertSxfCount > 0) {
            log.info("【用户平仓】保存明细记录成功");
        } else {
            log.error("用户平仓】保存明细记录出错");
            throw new Exception("用户平仓】保存明细记录出错");
        }
        return ServerResponse.createBySuccessMsg("平仓成功！");
    }

    public ServerResponse lock(Integer positionId, Integer state, String lockMsg) {
        if (positionId == null || state == null) {
            return ServerResponse.createByErrorMsg("参数不能为空");
        }

        UserPosition position = this.userPositionMapper.selectByPrimaryKey(positionId);
        if (position == null) {
            return ServerResponse.createByErrorMsg("持仓不存在");
        }

        if (position.getSellOrderId() != null) {
            return ServerResponse.createByErrorMsg("平仓单不能锁仓");
        }

        if (state.intValue() == 1 &&
                StringUtils.isBlank(lockMsg)) {
            return ServerResponse.createByErrorMsg("锁仓提示信息必填");
        }


        if (state.intValue() == 1) {
            position.setIsLock(Integer.valueOf(1));
            position.setLockMsg(lockMsg);
        } else {
            position.setIsLock(Integer.valueOf(0));
        }

        int updateCount = this.userPositionMapper.updateByPrimaryKeySelective(position);
        if (updateCount > 0) {
            return ServerResponse.createBySuccessMsg("操作成功");
        }
        return ServerResponse.createByErrorMsg("操作失败");
    }

    public ServerResponse del(Integer positionId) {
        if (positionId == null) {
            return ServerResponse.createByErrorMsg("id不能为空");
        }
        UserPosition position = this.userPositionMapper.selectByPrimaryKey(positionId);
        if (position == null) {
            ServerResponse.createByErrorMsg("该持仓不存在");
        }
        if (position.getSellOrderId() == null) {
            return ServerResponse.createByErrorMsg("持仓单不能删除！");
        }
        int updateCount = this.userPositionMapper.deleteByPrimaryKey(positionId);
        if (updateCount > 0) {
            return ServerResponse.createBySuccessMsg("删除成功");
        }
        return ServerResponse.createByErrorMsg("删除失败");
    }

    public ServerResponse findMyPositionByCodeAndSpell(String stockCode, String stockSpell, Integer state, HttpServletRequest request, int pageNum, int pageSize) {
        // 获取货币当前价格
        List<IndexInfo> indexInfoList = stockMapper.selectStockList();
        Map<String, IndexInfo> indexMap = new HashMap<>();
        BigDecimal nowPrice;
        for (IndexInfo indexInfo : indexInfoList) {
            if (StringUtils.isNotEmpty(indexInfo.getDataDbName())) {
                try {
                    StockListVO stockListVO = dbStockService.getStockListVO(indexInfo.getCode());
                    nowPrice = new BigDecimal(stockListVO.getNowPrice());
                    indexInfo.setNowPrice(nowPrice);
                } catch (Exception e) {
                    // 数据库不存在
                    log.info("数据库不存在");
                }
            }
            indexMap.put(indexInfo.getCnName(), indexInfo);
        }
        User user = this.iUserService.getCurrentRefreshUser(request);
        PageHelper.startPage(pageNum, pageSize);
//        BigDecimal totalassets=new BigDecimal(0);
        List<UserPosition> userPositions = this.userPositionMapper.findMyPositionByCodeAndSpell(user.getId(), stockCode, stockSpell, state);
        List<UserPositionVO> userPositionVOS = Lists.newArrayList();
        BigDecimal ykSum = new BigDecimal(0);
        for (UserPosition position : userPositions) {
            UserPositionVO userPositionVO = assembleUserPositionVO(position);
            IndexInfo indexInfo = indexMap.get(position.getStockName());
            userPositionVO.setNow_price(indexInfo.getNowPrice().toString());
            if (1 != state) {
                BigDecimal yk = new BigDecimal(0);
                //盈亏公式 做多 (现价—开仓价)*合约单位*手数   做空 （开仓价-现价）*合约单位*手数
                if ("做多".equals(position.getOrderDirection())) {
                    yk = indexInfo.getNowPrice().subtract(position.getBuyOrderPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
                } else {
                    yk = position.getBuyOrderPrice().subtract(indexInfo.getNowPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
                }
//                ServerResponse stock_res = this.iStockService.findStockByName(position.getStockName());
//                if (!stock_res.isSuccess()) {
//                    return ServerResponse.createByErrorMsg("货币代码错误");
//                }
//                Stock stock = (Stock) stock_res.getData();
//                if (stock.getIsLock().intValue() != 0) {
//                    return ServerResponse.createByErrorMsg("当前货币不能交易");
//                }
//                yk = yk.subtract(position.getOrderSpread());
                userPositionVO.setAllProfitAndLose(yk.add(userPositionVO.getOrderFee()));
                userPositionVO.setProfitAndLose(yk);
            }
            userPositionVOS.add(userPositionVO);
        }
        //可用余额
        BigDecimal enableAmt = user.getEnableAmt();
        //冻结保证金
        BigDecimal depositFreezeAmt = new BigDecimal(0);
        // 手续费
        BigDecimal sxfSum = new BigDecimal(0);
        List<UserPosition> userPositionAll = this.userPositionMapper.findMyPositionByCodeAndSpell(user.getId(), null, null, 0);
        for (UserPosition position : userPositionAll) {
            depositFreezeAmt = depositFreezeAmt.add(position.getDepositFreezeAmt());
            sxfSum = sxfSum.add(position.getOrderFee());
            //风险率
            IndexInfo indexInfo = indexMap.get(position.getStockName());
            BigDecimal yk = new BigDecimal(0);
            //盈亏公式 做多 (现价—开仓价)*合约单位*手数 - 点差   做空 （开仓价-现价）*合约单位*手数 - 点差
            if ("做多".equals(position.getOrderDirection())) {
                yk = indexInfo.getNowPrice().subtract(position.getBuyOrderPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
            } else {
                yk = position.getBuyOrderPrice().subtract(indexInfo.getNowPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
            }
//            yk = yk.subtract(position.getOrderSpread());
            ykSum = ykSum.add(yk);
        }
        BigDecimal hazardRate = new BigDecimal(0);
        if (0 != userPositionAll.size()) {
            hazardRate = hazardRate.add(enableAmt).add(depositFreezeAmt).add(ykSum).subtract(sxfSum);
            hazardRate = hazardRate.divide(depositFreezeAmt, 2, BigDecimal.ROUND_HALF_UP);
        }
        userPositionAll = this.userPositionMapper.findMyPositionByCodeAndSpell(user.getId(), null, null, 2);
        for (UserPosition position : userPositionAll) {
            depositFreezeAmt = depositFreezeAmt.add(position.getDepositFreezeAmt());
        }
        if (hazardRate.compareTo(BigDecimal.ZERO) == -1) {
            hazardRate = BigDecimal.ZERO;
        }
        PageInfo pageInfo = new PageInfo(userPositions);
        pageInfo.setList(userPositionVOS);
        Map<String, Object> map = new HashMap<>();
        map.put("pageInfo", pageInfo);
        map.put("hazardRate", hazardRate.multiply(new BigDecimal(100)));
        map.put("ykSum", ykSum);
        map.put("enableAmt", enableAmt);
        map.put("depositFreezeAmt", depositFreezeAmt);
        return ServerResponse.createBySuccess(map);
    }

    public PositionVO findUserPositionAllProfitAndLose(Integer userId) {
        // 获取货币当前价格
        List<IndexInfo> indexInfoList = stockMapper.selectStockList();
        Map<String, IndexInfo> indexMap = new HashMap<>();
        BigDecimal nowPrice;
        for (IndexInfo indexInfo : indexInfoList) {
            if (StringUtils.isNotEmpty(indexInfo.getDataDbName())) {
                try {
                    StockListVO stockListVO = dbStockService.getStockListVO(indexInfo.getCode());
                    nowPrice = new BigDecimal(stockListVO.getNowPrice());
                    indexInfo.setNowPrice(nowPrice);
                } catch (Exception e) {
                    // 数据库不存在
                    log.info("数据库不存在");
                }
            }
            indexMap.put(indexInfo.getCnName(), indexInfo);
        }
        List<UserPosition> userPositionAll = this.userPositionMapper.findMyPositionByCodeAndSpell(userId, null, null, 0);
        BigDecimal depositFreezeAmt = new BigDecimal(0);
        BigDecimal ykSum = new BigDecimal(0);
        for (UserPosition position : userPositionAll) {
            depositFreezeAmt = depositFreezeAmt.add(position.getDepositFreezeAmt());
            IndexInfo indexInfo = indexMap.get(position.getStockName());
            BigDecimal yk = new BigDecimal(0);
            //盈亏公式 做多 (现价—开仓价)*合约单位*手数 - 点差   做空 （开仓价-现价）*合约单位*手数 - 点差
            if ("做多".equals(position.getOrderDirection())) {
                yk = indexInfo.getNowPrice().subtract(position.getBuyOrderPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
            } else {
                yk = position.getBuyOrderPrice().subtract(indexInfo.getNowPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
            }
//            yk = yk.subtract(position.getOrderSpread());
            ykSum = ykSum.add(yk);
        }
        PositionVO positionVO = new PositionVO();
        positionVO.setAllProfitAndLose(ykSum);
        positionVO.setAllFreezAmt(depositFreezeAmt);
        return positionVO;
    }

    public List<UserPosition> findPositionByUserIdAndSellIdIsNull(Integer userId) {
        return this.userPositionMapper.findPositionByUserIdAndSellIdIsNull(userId);
    }

    public List<UserPosition> findPositionByStockCodeAndTimes(int minuteTimes, String stockCode, Integer userId) {
        Date paramTimes = null;
        paramTimes = DateTimeUtil.parseToDateByMinute(minuteTimes);

        return this.userPositionMapper.findPositionByStockCodeAndTimes(paramTimes, stockCode, userId);
    }

    public Integer findPositionNumByTimes(int minuteTimes, Integer userId) {
        Date beginDate = DateTimeUtil.parseToDateByMinute(minuteTimes);
        Integer transNum = this.userPositionMapper.findPositionNumByTimes(beginDate, userId);
        log.info("用户 {} 在 {} 分钟之内 交易手数 {}", new Object[]{userId, Integer.valueOf(minuteTimes), transNum});
        return transNum;
    }

    public ServerResponse listByAgent(Integer positionType, Integer state, String nickName, Integer agentId, String positionSn, String beginTime, String endTime, HttpServletRequest request, int pageNum, int pageSize) {
        AgentUser currentAgent = this.iAgentUserService.getCurrentAgent(request);
        if (agentId != null) {
            AgentUser agentUser = this.agentUserMapper.selectByPrimaryKey(agentId);
            if (agentUser.getParentId() != currentAgent.getId()) {
                return ServerResponse.createByErrorMsg("不能查询非下级代理用户持仓");
            }
        }
        Integer searchId = null;
        if (agentId == null) {
            searchId = currentAgent.getId();
        } else {
            searchId = agentId;
        }
        Timestamp begin_time = null;
        if (StringUtils.isNotBlank(beginTime)) {
            begin_time = DateTimeUtil.searchStrToTimestamp(beginTime);
        }
        Timestamp end_time = null;
        if (StringUtils.isNotBlank(endTime)) {
            end_time = DateTimeUtil.searchStrToTimestamp(endTime);
        }
        PageHelper.startPage(pageNum, pageSize);
        Map<String, IndexInfo> indexMap = new HashMap<>();
        List<IndexInfo> indexInfoList = stockMapper.selectStockList();
        for (IndexInfo indexInfo : indexInfoList) {
            if (StringUtils.isNotEmpty(indexInfo.getDataDbName())) {
                try {
                    StockListVO stockListVO = dbStockService.getStockListVO(indexInfo.getCode());
                    BigDecimal nowPrice = new BigDecimal(stockListVO.getNowPrice());
                    indexInfo.setNowPrice(nowPrice);
                } catch (Exception e) {
                    // 数据库不存在
                    log.info("数据库不存在");
                }
            }
            indexMap.put(indexInfo.getCnName(), indexInfo);
        }
        List<UserPosition> userPositions = this.userPositionMapper.listByAgent(positionType, state, nickName, searchId, positionSn, begin_time, end_time);
        List<AgentPositionVO> agentPositionVOS = Lists.newArrayList();
        for (UserPosition position : userPositions) {
            AgentPositionVO agentPositionVO = assembleAgentPositionVO(position, state, indexMap);
            agentPositionVOS.add(agentPositionVO);
        }
        PageInfo pageInfo = new PageInfo(userPositions);
        pageInfo.setList(agentPositionVOS);
        return ServerResponse.createBySuccess(pageInfo);
    }

    public ServerResponse getIncome(Integer agentId, Integer positionType, String beginTime, String endTime) {
        Timestamp begin_time = null;
        if (StringUtils.isNotBlank(beginTime)) {
            begin_time = DateTimeUtil.searchStrToTimestamp(beginTime);
        }
        Timestamp end_time = null;
        if (StringUtils.isNotBlank(endTime)) {
            end_time = DateTimeUtil.searchStrToTimestamp(endTime);
        }

        List<UserPosition> userPositions = this.userPositionMapper.listByAgent(positionType, null, null, agentId, null, begin_time, end_time);

        BigDecimal order_fee_amt = new BigDecimal("0");
        BigDecimal order_profit_and_lose = new BigDecimal("0");
        BigDecimal order_profit_and_lose_all = new BigDecimal("0");
        for (UserPosition position : userPositions) {
            // 挂单的不计算
            if (1 != position.getEntryOrders()) {
                order_fee_amt = order_fee_amt.add(position.getOrderFee()).add(position.getOrderSpread()).add(position.getOrderStayFee());
                // 平仓的数据 直接用数据库中的数据
                if (StringUtils.isNotEmpty(position.getSellOrderId())) {
                    order_profit_and_lose = order_profit_and_lose.add(position.getProfitAndLose());
                    order_profit_and_lose_all = order_profit_and_lose_all.add(position.getAllProfitAndLose());
                } else {
                    // 持仓的才计算实际的
                    List<IndexInfo> indexInfoList = stockMapper.selectStockList();
                    BigDecimal nowPrice;
                    Map<String, IndexInfo> indexMap = new HashMap<>();
                    for (IndexInfo indexInfo : indexInfoList) {
                        if (StringUtils.isNotEmpty(indexInfo.getDataDbName())) {
                            try {
                                StockListVO stockListVO = dbStockService.getStockListVO(indexInfo.getCode());
                                nowPrice = new BigDecimal(stockListVO.getNowPrice());
                                indexInfo.setNowPrice(nowPrice);
                            } catch (Exception e) {
                                // 数据库不存在
                                log.info("数据库不存在");
                            }
                        }
                        indexMap.put(indexInfo.getCnName(), indexInfo);
                    }
                    BigDecimal ykSum = new BigDecimal(0);
                    IndexInfo indexInfo = indexMap.get(position.getStockName());
                    BigDecimal yk = new BigDecimal(0);
                    //盈亏公式 做多 (现价—开仓价)*合约单位*手数   做空 （开仓价-现价）*合约单位*手数
                    if ("做多".equals(position.getOrderDirection())) {
                        yk = indexInfo.getNowPrice().subtract(position.getBuyOrderPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
                    } else {
                        yk = position.getBuyOrderPrice().subtract(indexInfo.getNowPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
                    }
//                    yk = yk.subtract(position.getOrderSpread());
                    order_profit_and_lose = order_profit_and_lose.add(yk);
                    order_profit_and_lose_all = order_profit_and_lose_all.add(yk);
                }
            }
        }
        AgentIncomeVO agentIncomeVO = new AgentIncomeVO();
        agentIncomeVO.setOrderSize(Integer.valueOf(userPositions.size()));
        agentIncomeVO.setOrderFeeAmt(order_fee_amt);
        agentIncomeVO.setOrderProfitAndLose(order_profit_and_lose);
        agentIncomeVO.setOrderAllAmt(order_profit_and_lose_all);
        return ServerResponse.createBySuccess(agentIncomeVO);
    }

    public ServerResponse listByAdmin(Integer agentId, Integer positionType, Integer state, String nickName, String positionSn, String beginTime, String endTime, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        Timestamp begin_time = null;
        if (StringUtils.isNotBlank(beginTime)) {
            begin_time = DateTimeUtil.searchStrToTimestamp(beginTime);
        }
        Timestamp end_time = null;
        if (StringUtils.isNotBlank(endTime)) {
            end_time = DateTimeUtil.searchStrToTimestamp(endTime);
        }
        List<UserPosition> userPositions = this.userPositionMapper.listByAgent(positionType, state, nickName, agentId, positionSn, begin_time, end_time);
        Map<String, IndexInfo> indexMap = new HashMap<>();
        List<IndexInfo> indexInfoList = stockMapper.selectStockList();
        for (IndexInfo indexInfo : indexInfoList) {
            if (StringUtils.isNotEmpty(indexInfo.getDataDbName())) {
                try {
                    StockListVO stockListVO = dbStockService.getStockListVO(indexInfo.getCode());
                    BigDecimal nowPrice = new BigDecimal(stockListVO.getNowPrice());
                    indexInfo.setNowPrice(nowPrice);
                } catch (Exception e) {
                    // 数据库不存在
                    log.info("数据库不存在");
                }
            }
            indexMap.put(indexInfo.getCnName(), indexInfo);
        }
        List<AdminPositionVO> adminPositionVOS = Lists.newArrayList();
        for (UserPosition position : userPositions) {
            IndexInfo indexInfo = indexMap.get(position.getStockName());
            AdminPositionVO adminPositionVO = assembleAdminPositionVO(position, state, indexMap);
            adminPositionVOS.add(adminPositionVO);
        }
        PageInfo pageInfo = new PageInfo(userPositions);
        pageInfo.setList(adminPositionVOS);
        return ServerResponse.createBySuccess(pageInfo);
    }

    public int CountPositionNum(Integer state, Integer accountType) {
        return this.userPositionMapper.CountPositionNum(state, accountType);
    }

    public BigDecimal CountPositionProfitAndLose() {
        return this.userPositionMapper.CountPositionProfitAndLose();
    }

    public BigDecimal CountPositionAllProfitAndLose() {
        return this.userPositionMapper.CountPositionAllProfitAndLose();
    }

    public ServerResponse create(Integer userId, String stockCode, String buyPrice, String buyTime, Integer buyNum, Integer buyType, BigDecimal lever) {
        if (userId == null || StringUtils.isBlank(buyPrice) || StringUtils.isBlank(stockCode) ||
                StringUtils.isBlank(buyTime) || buyNum == null || buyType == null || lever == null) {

            return ServerResponse.createByErrorMsg("参数不能为空");
        }
        User user = this.userMapper.selectByPrimaryKey(userId);
        if (user == null) {
            return ServerResponse.createByErrorMsg("用户不存在");
        }
        if (user.getAccountType().intValue() != 1) {
            return ServerResponse.createByErrorMsg("正式用户不能生成持仓单");
        }
        Stock stock = (Stock) this.iStockService.findStockByCode(stockCode).getData();
        if (stock == null) {
            return ServerResponse.createByErrorMsg("货币不存在");
        }
        SiteSetting siteSetting = this.iSiteSettingService.getSiteSetting();
        if (siteSetting == null) {
            log.error("下单出错，网站设置表不存在");
            return ServerResponse.createByErrorMsg("下单失败，系统设置错误");
        }
        BigDecimal user_enable_amt = user.getEnableAmt();
        BigDecimal buy_amt = (new BigDecimal(buyPrice)).multiply(new BigDecimal(buyNum.intValue()));
        BigDecimal buy_amt_autual = buy_amt.divide(new BigDecimal(lever.intValue()), 2, 4);
        int compareUserAmtInt = user_enable_amt.compareTo(buy_amt_autual);
        log.info("用户可用金额 = {}  实际购买金额 =  {}", user_enable_amt, buy_amt_autual);
        log.info("比较 用户金额 和 实际 购买金额 =  {}", Integer.valueOf(compareUserAmtInt));
        if (compareUserAmtInt == -1) {
            return ServerResponse.createByErrorMsg("下单失败，用户可用金额小于" + buy_amt_autual + "元");
        }
        BigDecimal reckon_enable = user_enable_amt.subtract(buy_amt_autual);
        user.setEnableAmt(reckon_enable);
        int updateUserCount = this.userMapper.updateByPrimaryKeySelective(user);
        if (updateUserCount > 0) {
            log.info("【用户交易下单】修改用户金额成功");
        } else {
            log.error("用户交易下单】修改用户金额出错");
        }
        UserPosition userPosition = new UserPosition();
        userPosition.setPositionType(Integer.valueOf(1));
        userPosition.setPositionSn(KeyUtils.getUniqueKey());
        userPosition.setUserId(user.getId());
        userPosition.setNickName(user.getRealName());
        userPosition.setAgentId(user.getAgentId());
        userPosition.setAgentName(user.getAgentName());
        userPosition.setStockCode(stock.getStockCode());
        userPosition.setStockName(stock.getStockName());
        userPosition.setStockGid(stock.getStockGid());
        userPosition.setStockSpell(stock.getStockSpell());
        userPosition.setBuyOrderId(GeneratePosition.getPositionId());
        userPosition.setBuyOrderTime(DateTimeUtil.strToDate(buyTime));
        userPosition.setBuyOrderPrice(new BigDecimal(buyPrice));
        userPosition.setOrderDirection((buyType == 0) ? "做多" : "做空");
        userPosition.setOrderNum(buyNum);
        userPosition.setIsLock(0);
        userPosition.setOrderLever(lever);
        userPosition.setOrderTotalPrice(buy_amt);
        BigDecimal buy_fee_amt = buy_amt.multiply(siteSetting.getBuyFee()).setScale(2, 4);
        log.info("创建模拟持仓 手续费（配资后总资金 * 百分比） = {}", buy_fee_amt);
        userPosition.setOrderFee(buy_fee_amt);
        BigDecimal buy_yhs_amt = buy_amt.multiply(siteSetting.getDutyFee()).setScale(2, 4);
        log.info("创建模拟持仓 点差（配资后总资金 * 百分比） = {}", buy_yhs_amt);
        userPosition.setOrderSpread(buy_yhs_amt);
        BigDecimal profit_and_lose = new BigDecimal("0");
        userPosition.setProfitAndLose(profit_and_lose);
        BigDecimal all_profit_and_lose = profit_and_lose.subtract(buy_fee_amt).subtract(buy_yhs_amt);
        userPosition.setAllProfitAndLose(all_profit_and_lose);
        userPosition.setOrderStayDays(0);
        userPosition.setOrderStayFee(new BigDecimal("0"));
        int insertPositionCount = this.userPositionMapper.insert(userPosition);
        if (insertPositionCount > 0) {
            log.info("【创建模拟持仓】保存记录成功");
        } else {
            log.error("【创建模拟持仓】保存记录出错");
        }
        return ServerResponse.createBySuccess("生成模拟持仓成功");
    }

    public int deleteByUserId(Integer userId) {
        return this.userPositionMapper.deleteByUserId(userId);
    }

    public void doClosingStayTask() {
        List<UserPosition> userPositions = this.userPositionMapper.findAllStayPosition();
        if (userPositions.size() > 0) {
            log.info("查询到正在持仓的订单数量 = {}", userPositions.size());
            for (UserPosition position : userPositions) {
                int stayDays = GetStayDays.getDays(GetStayDays.getBeginDate(position.getBuyOrderTime()));
                log.info("");
                log.info("开始处理 持仓订单id = {} 订单号 = {} 用户id = {} realName = {} 留仓天数 = {}", position
                        .getId(), position.getPositionSn(), position.getUserId(), position
                        .getNickName(), stayDays);
                if (stayDays != 0) {
                    log.info(" 开始收取 {} 天 留仓费", stayDays);
                    try {
                        closingStayTask(position, stayDays);
                    } catch (Exception e) {
                        log.error("doClosingStayTask = ", e);
                    }
                } else {
                    log.info("持仓订单 = {} ,持仓天数0天,不需要处理...", position.getId());
                }
                log.info("修改留仓费 处理结束。");
            }
            SiteTaskLog stl = new SiteTaskLog();
            stl.setTaskType("扣除留仓费");
            stl.setAddTime(new Date());
            stl.setIsSuccess(0);
            stl.setTaskTarget("扣除留仓费，订单数量为" + userPositions.size());
            this.siteTaskLogMapper.insert(stl);
        } else {
            log.info("doClosingStayTask没有正在持仓的订单");
        }
    }

    @Transactional
    public ServerResponse closingStayTask(UserPosition position, Integer stayDays) throws Exception {
        log.info("=================closingStayTask====================");
        log.info("修改留仓费，持仓id={},持仓天数={}", position.getId(), stayDays);
        SiteSetting siteSetting = this.iSiteSettingService.getSiteSetting();
        if (siteSetting == null) {
            log.error("修改留仓费出错，网站设置表不存在");
            return ServerResponse.createByErrorMsg("修改留仓费出错，网站设置表不存在");
        }
        BigDecimal stayFee = position.getOrderTotalPrice().multiply(siteSetting.getStayFee());
        BigDecimal allStayFee = stayFee.multiply(new BigDecimal(stayDays.intValue()));
        log.info("总留仓费 = {}", allStayFee);
        position.setOrderStayFee(allStayFee);
        position.setOrderStayDays(stayDays);
        BigDecimal all_profit = position.getAllProfitAndLose().subtract(allStayFee);
        position.setAllProfitAndLose(all_profit);
        int updateCount = this.userPositionMapper.updateByPrimaryKeySelective(position);
        if (updateCount > 0) {
            User user = this.userMapper.selectByPrimaryKey(position.getUserId());
            UserCashDetail ucd = new UserCashDetail();
            ucd.setPositionId(position.getId());
            ucd.setAgentId(user.getAgentId());
            ucd.setAgentName(user.getAgentName());
            ucd.setUserId(user.getId());
            ucd.setUserName(user.getRealName());
            ucd.setDeType("留仓费");
            ucd.setDeAmt(all_profit);
            ucd.setDeSummary("货币，" + position.getStockCode() + "/" + position.getStockName() + "，扣除留仓费：" + all_profit);
            ucd.setAddTime(new Date());
            ucd.setIsRead(Integer.valueOf(0));
            int insertSxfCount = this.userCashDetailMapper.insert(ucd);
            log.info("【closingStayTask收持仓费】修改持仓记录成功");
        } else {
            log.error("【closingStayTask收持仓费】修改持仓记录出错");
            throw new Exception("【closingStayTask收持仓费】修改持仓记录出错");
        }
        log.info("=======================================================");
        return ServerResponse.createBySuccess();
    }

    public List<Integer> findDistinctUserIdList() {
        return this.userPositionMapper.findDistinctUserIdList();
    }

    private AdminPositionVO assembleAdminPositionVO(UserPosition position, Integer state, Map<String, IndexInfo> indexMap) {
        AdminPositionVO adminPositionVO = new AdminPositionVO();
        adminPositionVO.setId(position.getId());
        adminPositionVO.setPositionSn(position.getPositionSn());
        adminPositionVO.setPositionType(position.getPositionType());
        adminPositionVO.setUserId(position.getUserId());
        adminPositionVO.setNickName(position.getNickName());
        adminPositionVO.setAgentId(position.getAgentId());
        adminPositionVO.setAgentName(position.getAgentName());
        adminPositionVO.setStockName(position.getStockName());
        adminPositionVO.setStockCode(position.getStockCode());
        adminPositionVO.setStockGid(position.getStockGid());
        adminPositionVO.setStockSpell(position.getStockSpell());
        adminPositionVO.setBuyOrderId(position.getBuyOrderId());
        adminPositionVO.setBuyOrderTime(position.getBuyOrderTime());
        adminPositionVO.setBuyOrderPrice(position.getBuyOrderPrice());
        adminPositionVO.setSellOrderId(position.getSellOrderId());
        adminPositionVO.setSellOrderTime(position.getSellOrderTime());
        adminPositionVO.setSellOrderPrice(position.getSellOrderPrice());
        adminPositionVO.setOrderDirection(position.getOrderDirection());
        adminPositionVO.setOrderNum(position.getOrderNum());
        adminPositionVO.setOrderLever(position.getOrderLever());
        adminPositionVO.setOrderTotalPrice(position.getOrderTotalPrice());
        adminPositionVO.setOrderFee(position.getOrderFee());
        adminPositionVO.setOrderSpread(position.getOrderSpread());
        adminPositionVO.setOrderStayFee(position.getOrderStayFee());
        adminPositionVO.setOrderStayDays(position.getOrderStayDays());
        adminPositionVO.setIsLock(position.getIsLock());
        adminPositionVO.setLockMsg(position.getLockMsg());
        adminPositionVO.setStockPlate(position.getStockPlate());
        // 计算浮动盈亏  如果是平仓的就不需要计算了  直接去数据库中的
        if (1 != state) {
            PositionProfitVO positionProfitVO = getPositionProfitVO(position, indexMap);
            adminPositionVO.setProfitAndLose(positionProfitVO.getProfitAndLose());
            adminPositionVO.setAllProfitAndLose(positionProfitVO.getAllProfitAndLose());
            adminPositionVO.setNow_price(positionProfitVO.getNowPrice());
            adminPositionVO.setHazardRate(positionProfitVO.getHazardRate());
        } else {
            adminPositionVO.setProfitAndLose(position.getProfitAndLose());
            adminPositionVO.setAllProfitAndLose(position.getAllProfitAndLose());
        }
        adminPositionVO.setDepositFreezeAmt(position.getDepositFreezeAmt());
        adminPositionVO.setEntryOrders(position.getEntryOrders());
        return adminPositionVO;
    }

    private AgentPositionVO assembleAgentPositionVO(UserPosition position, Integer state, Map<String, IndexInfo> indexMap) {
        AgentPositionVO agentPositionVO = new AgentPositionVO();
        agentPositionVO.setId(position.getId());
        agentPositionVO.setPositionSn(position.getPositionSn());
        agentPositionVO.setPositionType(position.getPositionType());
        agentPositionVO.setUserId(position.getUserId());
        agentPositionVO.setNickName(position.getNickName());
        agentPositionVO.setAgentId(position.getAgentId());
        agentPositionVO.setStockName(position.getStockName());
        agentPositionVO.setStockCode(position.getStockCode());
        agentPositionVO.setStockGid(position.getStockGid());
        agentPositionVO.setStockSpell(position.getStockSpell());
        agentPositionVO.setBuyOrderId(position.getBuyOrderId());
        agentPositionVO.setBuyOrderTime(position.getBuyOrderTime());
        agentPositionVO.setBuyOrderPrice(position.getBuyOrderPrice());
        agentPositionVO.setSellOrderId(position.getSellOrderId());
        agentPositionVO.setSellOrderTime(position.getSellOrderTime());
        agentPositionVO.setSellOrderPrice(position.getSellOrderPrice());
        agentPositionVO.setOrderDirection(position.getOrderDirection());
        agentPositionVO.setOrderNum(position.getOrderNum());
        agentPositionVO.setOrderLever(position.getOrderLever());
        agentPositionVO.setOrderTotalPrice(position.getOrderTotalPrice());
        agentPositionVO.setOrderFee(position.getOrderFee());
        agentPositionVO.setOrderSpread(position.getOrderSpread());
        agentPositionVO.setOrderStayFee(position.getOrderStayFee());
        agentPositionVO.setOrderStayDays(position.getOrderStayDays());
        agentPositionVO.setIsLock(position.getIsLock());
        agentPositionVO.setLockMsg(position.getLockMsg());
        agentPositionVO.setStockPlate(position.getStockPlate());
//        PositionProfitVO positionProfitVO = getPositionProfitVO(position, indexMap);
        // 计算浮动盈亏  如果是平仓的就不需要计算了  直接去数据库中的
        if (1 != state) {
            PositionProfitVO positionProfitVO = getPositionProfitVO(position, indexMap);
            agentPositionVO.setProfitAndLose(positionProfitVO.getProfitAndLose());
            agentPositionVO.setAllProfitAndLose(positionProfitVO.getAllProfitAndLose());
            agentPositionVO.setNow_price(positionProfitVO.getNowPrice());
            agentPositionVO.setHazardRate(positionProfitVO.getHazardRate());
        } else {
            agentPositionVO.setProfitAndLose(position.getProfitAndLose());
            agentPositionVO.setAllProfitAndLose(position.getAllProfitAndLose());
        }
//        agentPositionVO.setProfitAndLose(positionProfitVO.getProfitAndLose());
//        agentPositionVO.setAllProfitAndLose(positionProfitVO.getAllProfitAndLose());
//        agentPositionVO.setNow_price(positionProfitVO.getNowPrice());
//        agentPositionVO.setHazardRate(positionProfitVO.getHazardRate());
        return agentPositionVO;
    }

    private UserPositionVO assembleUserPositionVO(UserPosition position) {
        UserPositionVO userPositionVO = new UserPositionVO();
        userPositionVO.setId(position.getId());
        userPositionVO.setPositionType(position.getPositionType());
        userPositionVO.setPositionSn(position.getPositionSn());
        userPositionVO.setUserId(position.getUserId());
        userPositionVO.setNickName(position.getNickName());
        userPositionVO.setAgentId(position.getAgentId());
        userPositionVO.setStockName(position.getStockName());
        userPositionVO.setStockCode(position.getStockCode());
        userPositionVO.setStockGid(position.getStockGid());
        userPositionVO.setStockSpell(position.getStockSpell());
        userPositionVO.setBuyOrderId(position.getBuyOrderId());
        userPositionVO.setBuyOrderTime(position.getBuyOrderTime());
        userPositionVO.setBuyOrderPrice(position.getBuyOrderPrice());
        userPositionVO.setSellOrderId(position.getSellOrderId());
        userPositionVO.setSellOrderTime(position.getSellOrderTime());
        userPositionVO.setSellOrderPrice(position.getSellOrderPrice());
        userPositionVO.setProfitTargetPrice(position.getProfitTargetPrice());
        userPositionVO.setStopTargetPrice(position.getStopTargetPrice());
        userPositionVO.setOrderDirection(position.getOrderDirection());
        userPositionVO.setOrderNum(position.getOrderNum());
        userPositionVO.setOrderLever(position.getOrderLever());
        userPositionVO.setOrderTotalPrice(position.getOrderTotalPrice());
        userPositionVO.setOrderFee(position.getOrderFee());
        userPositionVO.setOrderSpread(position.getOrderSpread());
        userPositionVO.setOrderStayFee(position.getOrderStayFee());
        userPositionVO.setOrderStayDays(position.getOrderStayDays());
        userPositionVO.setStoploss(position.getStoploss());
        userPositionVO.setTargetprofit(position.getTargetprofit());
        userPositionVO.setStockPlate(position.getStockPlate());
//        PositionProfitVO positionProfitVO = getPositionProfitVO(position);
        userPositionVO.setProfitAndLose(position.getProfitAndLose());
        userPositionVO.setAllProfitAndLose(position.getAllProfitAndLose());
//        userPositionVO.setNow_price(position.getNowPrice());
        userPositionVO.setDepositFreezeAmt(position.getDepositFreezeAmt());
        userPositionVO.setEntryOrders(position.getEntryOrders());
        return userPositionVO;
    }

    private PositionProfitVO getPositionProfitVO(UserPosition position, Map<String, IndexInfo> indexMap) {
        BigDecimal profitAndLose = new BigDecimal("0");
        BigDecimal allProfitAndLose = new BigDecimal("0");
        BigDecimal nowPrice = new BigDecimal(0);
        BigDecimal yk = new BigDecimal(0);
        IndexInfo indexInfo = indexMap.get(position.getStockName());
        nowPrice = indexInfo.getNowPrice();
        //盈亏公式 做多 (现价—开仓价)*合约单位*手数   做空 （开仓价-现价）*合约单位*手数
        if ("做多".equals(position.getOrderDirection())) {
            yk = indexInfo.getNowPrice().subtract(position.getBuyOrderPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
        } else {
            yk = position.getBuyOrderPrice().subtract(indexInfo.getNowPrice()).multiply(BigDecimal.valueOf(position.getOrderNum() * indexInfo.getContractSize()));
        }
//        yk = yk.subtract(position.getOrderSpread());
//        allProfitAndLose = yk.add(position.getOrderFee());
        allProfitAndLose = yk;
        profitAndLose = yk;
        // 计算风险率
        User user = userMapper.selectByPrimaryKey(position.getUserId());
        List<UserPosition> userPositionAll = this.userPositionMapper.findMyPositionByCodeAndSpell(user.getId(), null, null, 0);
        BigDecimal depositFreezeAmt = new BigDecimal(0);
        BigDecimal sxfSum = new BigDecimal(0);
        BigDecimal ykSum = new BigDecimal(0);
        for (UserPosition positionTemp : userPositionAll) {
            depositFreezeAmt = depositFreezeAmt.add(positionTemp.getDepositFreezeAmt());
            sxfSum = sxfSum.add(positionTemp.getOrderFee());
            //风险率
            indexInfo = indexMap.get(positionTemp.getStockName());
            yk = new BigDecimal(0);
            //盈亏公式 做多 (现价—开仓价)*合约单位*手数 - 点差   做空 （开仓价-现价）*合约单位*手数 - 点差
            if ("做多".equals(positionTemp.getOrderDirection())) {
                yk = indexInfo.getNowPrice().subtract(positionTemp.getBuyOrderPrice()).multiply(BigDecimal.valueOf(positionTemp.getOrderNum() * indexInfo.getContractSize()));
            } else {
                yk = positionTemp.getBuyOrderPrice().subtract(indexInfo.getNowPrice()).multiply(BigDecimal.valueOf(positionTemp.getOrderNum() * indexInfo.getContractSize()));
            }
//            yk = yk.subtract(positionTemp.getOrderSpread());
            ykSum = ykSum.add(yk);
        }
        BigDecimal hazardRate = new BigDecimal(0);
        if (0 != userPositionAll.size()) {
            hazardRate = hazardRate.add(user.getEnableAmt()).add(depositFreezeAmt).add(ykSum).subtract(sxfSum);
            hazardRate = hazardRate.divide(depositFreezeAmt, 2, BigDecimal.ROUND_HALF_UP);
            if (hazardRate.compareTo(BigDecimal.ZERO) == -1) {
                hazardRate = BigDecimal.ZERO;
            }
            hazardRate = hazardRate.multiply(new BigDecimal(100));
        }
        PositionProfitVO positionProfitVO = new PositionProfitVO();
        positionProfitVO.setProfitAndLose(profitAndLose);
        positionProfitVO.setAllProfitAndLose(allProfitAndLose);
        positionProfitVO.setNowPrice(nowPrice.toString());
        positionProfitVO.setHazardRate(hazardRate);
        return positionProfitVO;
    }

    //获取持仓数
    public Integer findUserPositionAmt(Integer userId, String stockCode) {
        List<UserPosition> userPositions = this.userPositionMapper.findPositionAmt(userId, stockCode);
        Integer allPositionAmt = 0;
        for (UserPosition position : userPositions) {
            allPositionAmt += position.getOrderNum();
        }
        return allPositionAmt;
    }

    public void stopProfitTargetTask(int id) {
        // 开始计算止盈操作
        try {
            toStopProfitTargetById(id);
        } catch (Exception e) {
            log.error("止盈操作异常:" + e);
        }
    }

//    public StockIndexDF getStockIndexDFByDbName(String dbName){
//        String dbTableName = IndexDbCreatedUtil.createdDbTableNameByTime(dbName, new Date());
//        StockIndexDF stockIndexDF;
//        try {
//            stockIndexDF = stockMapper.selectStockDateByDf(dbTableName);
//            if(null == stockIndexDF){
//                // 当月数据还没有 先获取上个月的
//                Calendar c = Calendar.getInstance();
//                c.setTime(new Date());
//                c.add(Calendar.MONTH, -1);
//                Date lastMonth = c.getTime();
//                dbTableName = IndexDbCreatedUtil.createdDbTableNameByTime(dbName, lastMonth);
//                stockIndexDF = stockMapper.selectStockDateByDf(dbTableName);
//            }
//            return stockIndexDF;
//        }catch (Exception e){
//            // 当月数据还没有 先获取上个月的
//            Calendar c = Calendar.getInstance();
//            c.setTime(new Date());
//            c.add(Calendar.MONTH, -1);
//            Date lastMonth = c.getTime();
//            dbTableName = IndexDbCreatedUtil.createdDbTableNameByTime(dbName, lastMonth);
//            stockIndexDF = stockMapper.selectStockDateByDf(dbTableName);
//            return stockIndexDF;
//        }
//    }
//
//    public String getNowPriceStrByDbName(String dbName){
//        String dbTableName = IndexDbCreatedUtil.createdDbTableNameByTime(dbName, new Date());
//        String nowPriceStr;
//        try {
//            nowPriceStr = stockMapper.selectNowPrice(dbTableName);
//            if(null == nowPriceStr){
//                // 当月数据还没有 先获取上个月的
//                Calendar c = Calendar.getInstance();
//                c.setTime(new Date());
//                c.add(Calendar.MONTH, -1);
//                Date lastMonth = c.getTime();
//                dbTableName = IndexDbCreatedUtil.createdDbTableNameByTime(dbName, lastMonth);
//                nowPriceStr = stockMapper.selectNowPrice(dbTableName);
//            }
//            return nowPriceStr;
//        }catch (Exception e){
//            Calendar c = Calendar.getInstance();
//            c.setTime(new Date());
//            c.add(Calendar.MONTH, -1);
//            Date lastMonth = c.getTime();
//            dbTableName = IndexDbCreatedUtil.createdDbTableNameByTime(dbName, lastMonth);
//            nowPriceStr = stockMapper.selectNowPrice(dbTableName);
//            return nowPriceStr;
//        }
//    }

    public Integer getTodayOrderNum(String stockName) {
        return userPositionMapper.selectTodayOrderNum(stockName);
    }
}

