package com.xc.utils.stock.sina;


import com.google.common.collect.Lists;
import com.xc.common.ServerResponse;
import com.xc.pojo.IndexInfo;
import com.xc.pojo.Stock;
import com.xc.pojo.StockIndexDF;
import com.xc.utils.HttpClientRequest;
import com.xc.utils.PropertiesUtil;
import com.xc.utils.redis.JsonUtil;
import com.xc.utils.stock.BuyAndSellUtils;
import com.xc.utils.stock.sina.vo.SinaStockMinData;
import com.xc.vo.stock.StockListVO;
import com.xc.vo.stock.StockVO;
import com.xc.vo.stock.k.MinDataVO;
import com.xc.vo.stock.k.echarts.EchartsDataVO;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.Random;

import org.apache.commons.lang3.StringUtils;
import org.codehaus.jackson.type.TypeReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class SinaStockApi {
    public static final String sina_url = PropertiesUtil.getProperty("sina.single.stock.url");
    private static final Logger log = LoggerFactory.getLogger(SinaStockApi.class);

    public static String getSinaStock(String stockGid) {
        String sina_result = "";
        try {
            sina_result = HttpClientRequest.doGet(sina_url + stockGid);
        } catch (Exception e) {
            log.error("获取货币行情出错，错误信息 = {}", e);
        }
        return sina_result.substring(sina_result.indexOf("=") + 2);
    }


    public static StockListVO assembleStockListVO(StockIndexDF stockIndexDF) {
        StockListVO stockListVO = new StockListVO();
        if(null == stockIndexDF){
            return stockListVO;
        }
        stockListVO.setName(stockIndexDF.getCnName());

        stockListVO.setNowPrice(stockIndexDF.getNow());

        if(StringUtils.isNotEmpty(stockIndexDF.getPctChange())) {
            stockListVO.setHcrate(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getPctChange())));
        }
        stockListVO.setToday_max(stockIndexDF.getHigh());
        stockListVO.setToday_min(stockIndexDF.getLow());
        stockListVO.setPreclose_px(stockIndexDF.getPreClose());
        stockListVO.setOpen_px(stockIndexDF.getOpen());
        return stockListVO;
    }

    public static StockVO assembleStockVO(StockIndexDF stockIndexDF, Integer orderNum) {
        StockVO stockVO = new StockVO();

        stockVO.setName(stockIndexDF.getCnName());

        stockVO.setNowPrice(stockIndexDF.getNow());

        stockVO.setHcrate(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getPctChange())));

        stockVO.setToday_max(stockIndexDF.getHigh());

        stockVO.setToday_min(stockIndexDF.getLow());

        stockVO.setPreclose_px(stockIndexDF.getPreClose());

        stockVO.setOpen_px(stockIndexDF.getOpen());

        Random random = new Random();
        if(null == orderNum || 0 == orderNum){
            // 如果今天没人买 自己随机一个数
            orderNum = random.nextInt(10);
        }
        // 自己算
        // 成交量
        // 当天成交手数 * W
        Integer businessAmount = orderNum * 10000;
        stockVO.setBusiness_amount(businessAmount.toString());

        // 成交额
        // 手数 * 现价 W
        // stockIndexDF.getNow()
        BigDecimal businessBalance = BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).multiply(new BigDecimal(businessAmount));
        stockVO.setBusiness_balance(businessBalance.toString());

        // stockIndexDF.getNow()
        // 买1是最高  基于现价 - 10以内随机数
        stockVO.setBuy5(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).subtract(new BigDecimal((Math.random()*(10 - 8) + 8)))));
        stockVO.setBuy4(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).subtract(new BigDecimal((Math.random()*(8 - 6) + 6)))));
        stockVO.setBuy3(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).subtract(new BigDecimal((Math.random()*(6 - 4) + 4)))));
        stockVO.setBuy2(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).subtract(new BigDecimal((Math.random()*(4 - 2) + 2)))));
        stockVO.setBuy1(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).subtract(new BigDecimal((Math.random()*(2 - 1) + 1)))));

        // 卖1是最低  基于现价 + 10以内随机数
        stockVO.setSell5(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).add(new BigDecimal((Math.random()*(10 - 8) + 8)))));
        stockVO.setSell4(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).add(new BigDecimal((Math.random()*(8 - 6) + 6)))));
        stockVO.setSell3(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).add(new BigDecimal((Math.random()*(6 - 4) + 4)))));
        stockVO.setSell2(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).add(new BigDecimal((Math.random()*(4 - 2) + 2)))));
        stockVO.setSell1(String.valueOf(BigDecimal.valueOf(Double.valueOf(stockIndexDF.getNow())).add(new BigDecimal((Math.random()*(2 - 1) + 1)))));

        // 手数 * W + 1000以内的随机数
        stockVO.setBuy1_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setBuy2_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setBuy3_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setBuy4_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setBuy5_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setSell1_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setSell2_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setSell3_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setSell4_num(String.valueOf(businessAmount + random.nextInt(1000)));
        stockVO.setSell5_num(String.valueOf(businessAmount + random.nextInt(1000)));

        return stockVO;
    }

    public static StockVO assembleStockVO(String sinaResult) {
        StockVO stockVO = new StockVO();

        String[] hqarr = sinaResult.split(",");

        stockVO.setName(hqarr[0]);

        stockVO.setNowPrice(hqarr[3]);

        BigDecimal chang_rate = new BigDecimal("0");
        if ((new BigDecimal(hqarr[2])).compareTo(new BigDecimal("0")) != 0) {

            chang_rate = (new BigDecimal(hqarr[3])).subtract(new BigDecimal(hqarr[2]));

            chang_rate = chang_rate.multiply(new BigDecimal("100")).divide(new BigDecimal(hqarr[2]), 2, RoundingMode.HALF_UP);
        }
        stockVO.setHcrate(chang_rate);

        stockVO.setToday_max(hqarr[4]);

        stockVO.setToday_min(hqarr[5]);

        stockVO.setBusiness_amount(hqarr[8]);

        stockVO.setBusiness_balance(hqarr[9]);

        stockVO.setPreclose_px(hqarr[2]);

        stockVO.setOpen_px(hqarr[1]);

        stockVO.setBuy1(hqarr[6]);
        stockVO.setBuy2(hqarr[13]);
        stockVO.setBuy3(hqarr[15]);
        stockVO.setBuy4(hqarr[17]);
        stockVO.setBuy5(hqarr[19]);

        stockVO.setSell1(hqarr[7]);
        stockVO.setSell2(hqarr[23]);
        stockVO.setSell3(hqarr[25]);
        stockVO.setSell4(hqarr[27]);
        stockVO.setSell5(hqarr[29]);

        stockVO.setBuy1_num(hqarr[10]);
        stockVO.setBuy2_num(hqarr[12]);
        stockVO.setBuy3_num(hqarr[14]);
        stockVO.setBuy4_num(hqarr[16]);
        stockVO.setBuy5_num(hqarr[18]);

        stockVO.setSell1_num(hqarr[20]);
        stockVO.setSell2_num(hqarr[22]);
        stockVO.setSell3_num(hqarr[24]);
        stockVO.setSell4_num(hqarr[26]);
        stockVO.setSell5_num(hqarr[28]);

        return stockVO;
    }


    public static ServerResponse<MinDataVO> getStockMinK(Stock stock, int time, int ma, int size) {
        int maxSize = Integer.parseInt(PropertiesUtil.getProperty("sina.k.min.max.size"));
        if (size > maxSize) {
            size = maxSize;
        }

        String minUrl = PropertiesUtil.getProperty("sina.k.min.url");
        minUrl = minUrl + "?symbol=" + stock.getStockGid() + "&scale=" + time + "&ma=" + ma + "&datalen=" + size;

        String hqstr = "";
        try {
            hqstr = HttpClientRequest.doGet(minUrl);
        } catch (Exception e) {
            log.error("获取货币K线分时图出错，错误信息 = {}", e);
        }

        log.info(" time = {} ma = {} size = {}", new Object[]{Integer.valueOf(time), Integer.valueOf(ma), Integer.valueOf(size)});


        hqstr = hqstr.replace("day", "\"day\"").replace("open", "\"open\"").replace("high", "\"high\"").replace("low", "\"low\"").replace("close", "\"close\"");

        if (ma == 5) {

            hqstr = hqstr.replace("ma_volume5", "\"ma_volume\"").replace(",volume", ",\"volume\"").replace("ma_price5", "\"ma_price\"");
        } else if (ma == 10) {


            hqstr = hqstr.replace("ma_volume10", "\"ma_volume\"").replace(",volume", ",\"volume\"").replace("ma_price10", "\"ma_price\"");
        } else if (ma == 15) {

            hqstr = hqstr.replace("ma_volume15", "\"ma_volume\"").replace(",volume", ",\"volume\"").replace("ma_price15", "\"ma_price\"");
        } else {
            return ServerResponse.createByErrorMsg("ma 取值 5，10，15");
        }


        if (StringUtils.isBlank(hqstr)) {
            return ServerResponse.createByErrorMsg("没有查询到行情数据");
        }

        MinDataVO minDataVO = new MinDataVO();
        minDataVO.setStockName(stock.getStockName());
        minDataVO.setStockCode(stock.getStockCode());
        minDataVO.setGid(stock.getStockGid());

        hqstr = hqstr.replaceAll("\"\"", "\"");

        List<SinaStockMinData> list = (List<SinaStockMinData>)JsonUtil.string2Obj(hqstr, new TypeReference<List<SinaStockMinData>>(){});
        log.info("需要查询的行情size为： {}", Integer.valueOf(size));

        minDataVO.setData(list);

        return ServerResponse.createBySuccess(minDataVO);
    }

    public static EchartsDataVO assembleEchartsDataVO(MinDataVO minDataVO) {
        EchartsDataVO echartsDataVO = new EchartsDataVO();
        echartsDataVO.setStockName(minDataVO.getStockName());
        echartsDataVO.setStockCode(minDataVO.getStockCode());

        List<SinaStockMinData> minDataList = minDataVO.getData();


        double[][] values = (double[][]) null;
        Object[][] volumes = (Object[][]) null;
        String[] date = null;

        if (minDataList.size() > 0) {

            values = new double[minDataList.size()][5];

            volumes = new Object[minDataList.size()][3];

            date = new String[minDataList.size()];

            for (int i = 0; i < minDataList.size(); i++) {
                SinaStockMinData sinaStockMinData = (SinaStockMinData) minDataList.get(i);

                for (int j = 0; j < values[i].length; j++) {
                    values[i][0] = Double.valueOf(sinaStockMinData.getOpen()).doubleValue();
                    values[i][1] = Double.valueOf(sinaStockMinData.getClose()).doubleValue();
                    values[i][2] = Double.valueOf(sinaStockMinData.getLow()).doubleValue();
                    values[i][3] = Double.valueOf(sinaStockMinData.getHigh()).doubleValue();
                    values[i][4] = Double.valueOf(sinaStockMinData.getVolume()).doubleValue();
                }
                for (int k = 0; k < volumes[i].length; k++) {
                    volumes[i][0] = Integer.valueOf(i);
                    volumes[i][1] = Double.valueOf(sinaStockMinData.getVolume());

                    if ((new BigDecimal(sinaStockMinData.getClose()))
                            .compareTo(new BigDecimal(sinaStockMinData.getOpen())) == 1) {
                        volumes[i][2] = Integer.valueOf(1);
                    } else {
                        volumes[i][2] = Integer.valueOf(-1);
                    }
                }

                date[i] = sinaStockMinData.getDay();
            }
        }

        echartsDataVO.setValues(values);
        echartsDataVO.setVolumes(volumes);
        echartsDataVO.setDate(date);

        return echartsDataVO;
    }


    public static void main(String[] args) {
//        List list = Lists.newArrayList();
//        list.add(Integer.valueOf(1));
//        list.add(Integer.valueOf(2));
//        list.add(Integer.valueOf(3));
//        System.out.println(list.size());
//
//        String[][] values = new String[list.size()][5];
//
//        System.out.println("[]" + values.length);
//        System.out.println("[][]" + values[1].length);
//
//        System.out.println(getSinaStock("sh601318"));
//
//        String sss = "[\n{\n\"day\": \"2019-03-05 14:50:00\",\n\"open\": \"13.020\",\n\"high\": \"13.040\",\n\"low\": \"13.000\",\n\"close\": \"13.040\",\n\"volume\": \"2611513\",\n\"ma_price5\": 13.01,\n\"ma_volume5\": 3216535\n},\n{\n\"day\": \"2019-03-05 14:55:00\",\n\"open\": \"13.040\",\n\"high\": \"13.040\",\n\"low\": \"13.010\",\n\"close\": \"13.030\",\n\"volume\": \"2296000\",\n\"ma_price5\": 13.016,\n\"ma_volume5\": 3044839\n}\n]";
//
//        sss = sss.substring(1, sss.length() - 1);
//
//        sss = "{" + sss + "}";
        getSinaStock("s_sh600090");
    }
}

