package com.xc.utils.thread;

import java.util.concurrent.*;

/**
 * 线程工具类
 */
public class ThreadUtil {
//    public static final ScheduledExecutorService SCHEDULED_EXECUTOR = Executors.newScheduledThreadPool(Runtime.getRuntime().availableProcessors()*2);
//    public static final ScheduledExecutorService SCHEDULED_EXECUTOR = new ScheduledThreadPoolExecutor(1,
//            new BasicThreadFactory.Builder().namingPattern("example-schedule-pool-%d").daemon(true).build());

    public static final ScheduledThreadPoolExecutor SCHEDULED_EXECUTOR = ThreadPool.getThreadPoolExecutor();
    
    /**
     * @Description: 立即执行线程
     * @param @param callback 参数
     */
    public static void executeInThread(Callback callback){
        SCHEDULED_EXECUTOR.execute(()->callback.execute(Thread.currentThread().getName()));
    }
    public static void executeInThread(Callback callback,Object obj){
        SCHEDULED_EXECUTOR.execute(()->callback.execute(obj));
    }

    /**
     * @Description: 延时执行线程
     * @param @param callback 延时 秒钟
     */
    public static void executeInThread(Callback callback,long delay){
        SCHEDULED_EXECUTOR.schedule(()-> callback.execute(
                Thread.currentThread().getName()),
                delay,
                TimeUnit.SECONDS);

    }

    /**
     * 定时执行自定义内容线程
     * @param callback 内容
     * @param initialDelay 初始化延时
     * @param period 两次开始执行最小间隔时间
     */
    public static void executeCallback(Callback callback,long initialDelay,long period){
        SCHEDULED_EXECUTOR.scheduleAtFixedRate(()->callback.execute(
                Thread.currentThread().getName()),
                initialDelay,
                period,
                TimeUnit.SECONDS);
    }

    /**
     *  定时执行线程
     * @param runnable 执行线程
     * @param initialDelay 初始化延时
     * @param period 两次开始执行最小间隔时间
     */
    public static void executeRunnable(Runnable runnable,long initialDelay,long period){
        SCHEDULED_EXECUTOR.scheduleAtFixedRate(runnable, initialDelay, period, TimeUnit.SECONDS);
    }

    /**
     *  定时执行线程
     * @param runnable 执行线程
     * @param initialDelay 初始化延时
     * @param period 两次开始执行最小间隔时间
     */
    public static void executeTimerTask(TimerTask runnable,long initialDelay,long period){
        SCHEDULED_EXECUTOR.scheduleAtFixedRate(runnable, initialDelay, period, TimeUnit.SECONDS);
    }

    public static void main(String[] args) {
        ThreadUtil.executeInThread(obj -> {
            System.out.println("6666");
        });
        ThreadUtil.executeCallback(obj -> {
            System.out.println("6666");
        },0,3000);
    }
}
